from ipam.models import Role
from tenancy.models import Tenant
from utilities.testing import APIViewTestCases  # , APITestCase, create_test_device

from netbox_routing.models.community import *

__all__ = (
    'CommunityTestCase',
    'CommunityListTestCase',
    'CommunityListEntryTestCase',
)


class CommunityTestCase(APIViewTestCases.APIViewTestCase):
    model = Community
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'community',
        'description',
        'display',
        'id',
        'url',
    ]

    bulk_update_data = {'status': 'reserved'}

    @classmethod
    def setUpTestData(cls):
        role = Role.objects.create(name='Test Role')
        communities = (
            Community(community='64512', status='active', role=role),
            Community(community='64513', status='active', role=role),
            Community(community='64514', status='active', role=role),
        )
        Community.objects.bulk_create(communities)

        cls.create_data = [
            {
                'community': '64512:1',
                'status': 'active',
                'role': role.pk,
            },
            {
                'community': '64512:2:3',
                'status': 'active',
                'role': role.pk,
            },
        ]


class CommunityListTestCase(APIViewTestCases.APIViewTestCase):
    model = CommunityList
    view_namespace = "plugins-api:netbox_routing"
    graphql_base_name = 'communitylist'
    brief_fields = [
        'description',
        'display',
        'id',
        'name',
        'url',
    ]

    bulk_update_data = {}

    @classmethod
    def setUpTestData(cls):
        community_lists = (
            CommunityList(
                name='Community List 1',
            ),
            CommunityList(
                name='Community List 2',
            ),
            CommunityList(
                name='Community List 3',
            ),
        )
        CommunityList.objects.bulk_create(community_lists)

        cls.create_data = [
            {
                'name': 'Community List 4',
            },
            {
                'name': 'Community List 5',
            },
        ]

        tenant = Tenant.objects.create(name='Test Tenant')
        cls.bulk_update_data = {'tenant': tenant.pk}


class CommunityListEntryTestCase(APIViewTestCases):
    model = CommunityListEntry
    view_namespace = "plugins-api:netbox_routing"
    graphql_base_name = 'communitylistentry'
    brief_fields = [
        'community',
        'community_list',
        'description',
        'display',
        'id',
        'url',
    ]

    bulk_update_data = {}

    @classmethod
    def setUpTestData(cls):
        tenant = Tenant.objects.create(name='Test Tenant', slug='test-tenant')
        role = Role.objects.create(name='Test Role', slug='test-role')
        communities = (
            Community(
                community='64512',
                status='active',
                role=role,
                tenant=tenant,
            ),
            Community(
                community='64513',
                status='active',
                role=role,
                tenant=tenant,
            ),
            Community(
                community='64514',
                status='active',
                role=role,
                tenant=tenant,
            ),
        )
        Community.objects.bulk_create(communities)
        community_lists = (
            CommunityList(
                name='Community List 1',
                tenant=tenant,
            ),
        )
        CommunityList.objects.bulk_create(community_lists)

        cls.create_data = [
            {
                'community_list': community_lists[0].pk,
                'community': communities[0].pk,
            },
            {
                'community_list': community_lists[0].pk,
                'community': communities[1].pk,
            },
            {
                'community_list': community_lists[0].pk,
                'community': communities[2].pk,
            },
        ]

        cls.bulk_update_data = {'description': 'Test updated description'}



from django.contrib.contenttypes.models import ContentType
from rest_framework import serializers

from netbox.api.fields import ContentTypeField
from netbox.api.gfk_fields import GFKSerializerField
from netbox.api.serializers import NetBoxModelSerializer
from netbox_routing.constants.objects import PREFIX_ASSIGNMENT_MODELS
from netbox_routing.models.objects import *
from netbox_routing.api._serializers.community import *

__all__ = (
    'ASPathSerializer',
    'ASPathEntrySerializer',
    'CustomPrefixSerializer',
    'PrefixListSerializer',
    'PrefixListEntrySerializer',
    'RouteMapSerializer',
    'RouteMapEntrySerializer',
)


class ASPathSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:aspath-detail'
    )

    class Meta:
        model = ASPath
        fields = (
            'url',
            'id',
            'display',
            'name',
            'description',
            'comments',
        )
        brief_fields = ('url', 'id', 'display', 'name')


class ASPathEntrySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:aspathentry-detail'
    )
    aspath = ASPathSerializer(nested=True)

    class Meta:
        model = ASPathEntry
        fields = (
            'url',
            'id',
            'display',
            'aspath',
            'sequence',
            'action',
            'pattern',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'aspath',
            'sequence',
            'action',
            'pattern',
        )


class PrefixListSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:prefixlist-detail'
    )

    class Meta:
        model = PrefixList
        fields = (
            'url',
            'id',
            'display',
            'name',
            'description',
            'comments',
            'custom_fields',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
        )


class PrefixListEntrySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:prefixlistentry-detail'
    )
    prefix_list = PrefixListSerializer(nested=True)
    assigned_prefix_type = ContentTypeField(
        queryset=ContentType.objects.filter(PREFIX_ASSIGNMENT_MODELS),
        allow_null=True,
        required=False,
        default=None,
    )
    assigned_prefix_id = serializers.IntegerField(
        allow_null=True, required=False, default=None
    )
    assigned_prefix = GFKSerializerField(read_only=True)

    class Meta:
        model = PrefixListEntry
        fields = (
            'url',
            'id',
            'display',
            'prefix_list',
            'sequence',
            'action',
            'assigned_prefix_type',
            'assigned_prefix_id',
            'assigned_prefix',
            'le',
            'ge',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'prefix_list',
            'sequence',
            'action',
            'assigned_prefix_type',
            'assigned_prefix_id',
            'le',
            'ge',
        )


class CustomPrefixSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:customprefix-detail'
    )

    class Meta:
        model = CustomPrefix
        fields = (
            'url',
            'id',
            'display',
            'prefix',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'prefix',
        )


class RouteMapSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:prefixlist-detail'
    )

    class Meta:
        model = RouteMap
        fields = (
            'url',
            'id',
            'display',
            'name',
            'description',
            'comments',
        )
        brief_fields = ('url', 'id', 'display', 'name')


class RouteMapEntrySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:prefixlistentry-detail'
    )
    route_map = RouteMapSerializer(nested=True)
    match_prefix_list = PrefixListSerializer(nested=True, many=True, required=False)
    match_community_list = CommunityListSerializer(
        nested=True, many=True, required=False
    )
    match_community = CommunitySerializer(nested=True, many=True, required=False)
    match_aspath = ASPathSerializer(nested=True, many=True, required=False)

    class Meta:
        model = RouteMapEntry
        fields = (
            'url',
            'id',
            'display',
            'route_map',
            'sequence',
            'action',
            'match_prefix_list',
            'match_community_list',
            'match_community',
            'match_aspath',
            'match',
            'set',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'route_map',
            'sequence',
            'action',
        )



from netbox.api.viewsets import NetBoxModelViewSet
from netbox_routing import filtersets
from netbox_routing.api.serializers import (
    EIGRPRouterSerializer,
    EIGRPAddressFamilySerializer,
    EIGRPInterfaceSerializer,
    EIGRPNetworkSerializer,
)
from netbox_routing.models import (
    EIGRPRouter,
    EIGRPAddressFamily,
    EIGRPNetwork,
    EIGRPInterface,
)

__all__ = (
    'EIGRPRouterViewSet',
    'EIGRPAddressFamilyViewSet',
    'EIGRPNetworkViewSet',
    'EIGRPInterfaceViewSet',
)


class EIGRPRouterViewSet(NetBoxModelViewSet):
    queryset = EIGRPRouter.objects.all()
    serializer_class = EIGRPRouterSerializer
    filterset_class = filtersets.EIGRPRouterFilterSet


class EIGRPAddressFamilyViewSet(NetBoxModelViewSet):
    queryset = EIGRPAddressFamily.objects.all()
    serializer_class = EIGRPAddressFamilySerializer
    filterset_class = filtersets.EIGRPAddressFamilyFilterSet


class EIGRPNetworkViewSet(NetBoxModelViewSet):
    queryset = EIGRPNetwork.objects.all()
    serializer_class = EIGRPNetworkSerializer
    filterset_class = filtersets.EIGRPNetworkFilterSet


class EIGRPInterfaceViewSet(NetBoxModelViewSet):
    queryset = EIGRPInterface.objects.all()
    serializer_class = EIGRPInterfaceSerializer
    filterset_class = filtersets.EIGRPInterfaceFilterSet



import netaddr

from ipam.models import VRF
from utilities.testing import ViewTestCases, create_test_device

from netbox_routing.models import StaticRoute

__all__ = ('StaticRouteTestCase',)


class StaticRouteTestCase(
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = StaticRoute

    @classmethod
    def setUpTestData(cls):
        devices = [
            create_test_device(name='Device 1'),
            create_test_device(name='Device 2'),
        ]
        vrf = VRF.objects.create(name='Test VRF')

        nh = netaddr.IPAddress('10.10.10.1')

        routes = (
            StaticRoute(name="Route 1", vrf=vrf, prefix='0.0.0.0/0', next_hop=nh),
            StaticRoute(
                name="Route 2",
                vrf=vrf,
                prefix='1.1.1.0/24',
                next_hop=netaddr.IPAddress('10.10.10.2'),
            ),
            StaticRoute(name="Route 3", prefix='0.0.0.0/0', next_hop=nh, metric=100),
        )

        StaticRoute.objects.bulk_create(routes)

        routes[0].devices.set([devices[0]])
        routes[1].devices.set([devices[0]])
        routes[2].devices.set([devices[0]])

        cls.form_data = {
            'name': 'Route X',
            'devices': [devices[1].pk],
            'vrf': vrf.pk,
            'prefix': netaddr.IPNetwork('1.1.1.1/32'),
            'next_hop': nh,
        }

        cls.bulk_edit_data = {
            'metric': 10,
        }

    def _get_base_url(self):
        return 'plugins:netbox_routing:staticroute_{}'



from netbox.views.generic import (
    ObjectListView,
    ObjectEditView,
    ObjectView,
    ObjectDeleteView,
    ObjectChildrenView,
    BulkImportView,
    BulkEditView,
    BulkDeleteView,
)
from netbox_routing.filtersets.ospf import (
    OSPFInterfaceFilterSet,
    OSPFAreaFilterSet,
    OSPFInstanceFilterSet,
)
from netbox_routing.forms import *
from netbox_routing.tables.ospf import (
    OSPFAreaTable,
    OSPFInstanceTable,
    OSPFInterfaceTable,
)
from utilities.views import register_model_view, ViewTab

from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface

__all__ = (
    'OSPFInstanceListView',
    'OSPFInstanceView',
    'OSPFInstanceInterfacesView',
    'OSPFInstanceEditView',
    'OSPFInstanceBulkEditView',
    'OSPFInstanceDeleteView',
    'OSPFInstanceBulkDeleteView',
    'OSPFAreaListView',
    'OSPFAreaView',
    'OSPFAreaInterfacesView',
    'OSPFAreaEditView',
    'OSPFAreaBulkEditView',
    'OSPFAreaDeleteView',
    'OSPFAreaBulkDeleteView',
    'OSPFInterfaceListView',
    'OSPFInterfaceView',
    'OSPFInterfaceEditView',
    'OSPFInterfaceBulkEditView',
    'OSPFInterfaceDeleteView',
    'OSPFInterfaceBulkDeleteView',
)


#
# Instance
#
@register_model_view(OSPFInstance, name='list', path='', detail=False)
class OSPFInstanceListView(ObjectListView):
    queryset = OSPFInstance.objects.all()
    table = OSPFInstanceTable
    filterset = OSPFInstanceFilterSet
    filterset_form = OSPFInstanceFilterForm


@register_model_view(OSPFInstance)
class OSPFInstanceView(ObjectView):
    queryset = OSPFInstance.objects.all()
    template_name = 'netbox_routing/ospfinstance.html'


@register_model_view(OSPFInstance, name='interfaces')
class OSPFInstanceInterfacesView(ObjectChildrenView):
    template_name = 'netbox_routing/ospf_interfaces.html'
    queryset = OSPFInstance.objects.all()
    child_model = OSPFInterface
    table = OSPFInterfaceTable
    filterset = OSPFInterfaceFilterSet
    tab = ViewTab(
        label='Interfaces',
        badge=lambda obj: OSPFInterface.objects.filter(instance=obj).count(),
        hide_if_empty=False,
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(instance=parent)

    def get_extra_context(self, request, instance):
        return {
            'form_interface_assignment': 'instance',
        }


@register_model_view(OSPFInstance, 'add', detail=False)
@register_model_view(OSPFInstance, name='edit')
class OSPFInstanceEditView(ObjectEditView):
    queryset = OSPFInstance.objects.all()
    form = OSPFInstanceForm


@register_model_view(OSPFInstance, name='delete')
class OSPFInstanceDeleteView(ObjectDeleteView):
    queryset = OSPFInstance.objects.all()


@register_model_view(OSPFInstance, name='bulk_edit', detail=False)
class OSPFInstanceBulkEditView(BulkEditView):
    queryset = OSPFInstance.objects.all()
    filterset = OSPFInstanceFilterSet
    table = OSPFInstanceTable
    form = OSPFInstanceBulkEditForm


@register_model_view(OSPFInstance, name='bulk_delete', detail=False)
class OSPFInstanceBulkDeleteView(BulkDeleteView):
    queryset = OSPFInstance.objects.all()
    filterset = OSPFInstanceFilterSet
    table = OSPFInstanceTable


@register_model_view(OSPFInstance, name='bulk_import', detail=False)
class OSPFInstanceBulkImportView(BulkImportView):
    queryset = OSPFInstance.objects.all()
    model_form = OSPFInstanceImportForm


#
# Area
#
@register_model_view(
    OSPFArea,
    name='list',
    path='',
    detail=False,
)
class OSPFAreaListView(ObjectListView):
    queryset = OSPFArea.objects.all()
    table = OSPFAreaTable
    filterset = OSPFAreaFilterSet
    filterset_form = OSPFAreaFilterForm


@register_model_view(OSPFArea)
class OSPFAreaView(ObjectView):
    queryset = OSPFArea.objects.all()
    template_name = 'netbox_routing/ospfarea.html'


@register_model_view(OSPFArea, name='interfaces')
class OSPFAreaInterfacesView(ObjectChildrenView):
    template_name = 'netbox_routing/ospf_interfaces.html'
    queryset = OSPFArea.objects.all()
    child_model = OSPFInterface
    table = OSPFInterfaceTable
    filterset = OSPFInterfaceFilterSet
    tab = ViewTab(
        label='Interfaces',
        badge=lambda obj: OSPFInterface.objects.filter(area=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(area=parent)

    def get_extra_context(self, request, instance):
        return {
            'form_interface_assignment': 'area',
        }


@register_model_view(OSPFArea, name='add', detail=False)
@register_model_view(OSPFArea, name='edit')
class OSPFAreaEditView(ObjectEditView):
    queryset = OSPFArea.objects.all()
    form = OSPFAreaForm


@register_model_view(OSPFArea, name='delete')
class OSPFAreaDeleteView(ObjectDeleteView):
    queryset = OSPFArea.objects.all()


@register_model_view(OSPFArea, name='bulk_edit', detail=False)
class OSPFAreaBulkEditView(BulkEditView):
    queryset = OSPFArea.objects.all()
    table = OSPFAreaTable
    filterset = OSPFAreaFilterSet
    form = OSPFAreaBulkEditForm


@register_model_view(OSPFArea, name='bulk_delete', detail=False)
class OSPFAreaBulkDeleteView(BulkDeleteView):
    queryset = OSPFArea.objects.all()
    table = OSPFAreaTable
    filterset = OSPFAreaFilterSet


@register_model_view(OSPFArea, name='bulk_import', detail=False)
class OSPFAreaBulkImportView(BulkImportView):
    queryset = OSPFArea.objects.all()
    model_form = OSPFAreaImportForm


#
# Interface
#
@register_model_view(OSPFInterface, name='list', path='', detail=False)
class OSPFInterfaceListView(ObjectListView):
    queryset = OSPFInterface.objects.all()
    table = OSPFInterfaceTable
    filterset = OSPFInterfaceFilterSet
    filterset_form = OSPFInterfaceFilterForm


@register_model_view(OSPFInterface)
class OSPFInterfaceView(ObjectView):
    queryset = OSPFInterface.objects.all()
    template_name = 'netbox_routing/ospfinterface.html'


@register_model_view(OSPFInterface, name='add', detail=False)
@register_model_view(OSPFInterface, name='edit')
class OSPFInterfaceEditView(ObjectEditView):
    queryset = OSPFInterface.objects.all()
    form = OSPFInterfaceForm


@register_model_view(OSPFInterface, name='delete')
class OSPFInterfaceDeleteView(ObjectDeleteView):
    queryset = OSPFInterface.objects.all()


@register_model_view(OSPFInterface, name='bulk_import', detail=False)
class OSPFInterfaceBulkImportView(BulkImportView):
    queryset = OSPFInterface.objects.all()
    model_form = OSPFInterfaceImportForm


@register_model_view(OSPFInterface, name='bulk_edit', detail=False)
class OSPFInterfaceBulkEditView(BulkEditView):
    queryset = OSPFInterface.objects.all()
    filterset = OSPFInterfaceFilterSet
    table = OSPFInterfaceTable
    form = OSPFInterfaceBulkEditForm


@register_model_view(OSPFInterface, name='bulk_delete', detail=False)
class OSPFInterfaceBulkDeleteView(BulkDeleteView):
    queryset = OSPFInterface.objects.all()
    filterset = OSPFInterfaceFilterSet
    table = OSPFInterfaceTable



from django.test import TestCase

from dcim.models import Device
from utilities.testing import create_test_device

from netbox_routing.forms import *

__all__ = ('StaticRouteTestCase',)


class StaticRouteTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.device = create_test_device(name='Device 1')

    def test_staticroute(self):
        form = StaticRouteForm(
            data={
                'name': 'Route 1',
                'devices': [Device.objects.first().pk],
                'vrf': None,
                'prefix': '0.0.0.0/0',
                'next_hop': '10.10.10.1',
                'metric': 1,
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())



from typing import Annotated, Union, List

import strawberry
import strawberry_django

from netbox.graphql.types import PrimaryObjectType

from netbox_routing import models
from netbox_routing.graphql.bgp.filters import (
    BGPSettingFilter,
    BGPPeerTemplateFilter,
    BGPPolicyTemplateFilter,
    BGPSessionTemplateFilter,
    BGPRouterFilter,
    BGPScopeFilter,
    BGPAddressFamilyFilter,
    BGPPeerFilter,
    BGPPeerAddressFamilyFilter,
    BFDProfileFilter,
)
from netbox_routing.graphql.objects.types import PrefixListType, RouteMapType

__all__ = (
    'BGPPeerTemplateType',
    'BGPPolicyTemplateType',
    'BGPSessionTemplateType',
    'BGPRouterType',
    'BGPScopeType',
    'BGPAddressFamilyType',
    'BGPPeerType',
    'BGPPeerAddressFamilyType',
    'BFDProfileType',
)

from netbox_routing.graphql.types_mixin import BGPSettingsMixin


@strawberry_django.type(
    models.BGPSetting,
    fields='__all__',
    exclude=['assigned_object_type', 'assigned_object_id'],
    filters=BGPSettingFilter,
)
class BGPSettingType(PrimaryObjectType):

    assigned_object_type: (
        Annotated["ContentTypeType", strawberry.lazy('netbox.graphql.types')] | None
    )  # noqa: F821
    assigned_object: Union[
        Annotated[
            'BGPPeerTemplateType', strawberry.lazy('netbox_routing.graphql.types')
        ],
        Annotated[
            'BGPPolicyTemplateType', strawberry.lazy('netbox_routing.graphql.types')
        ],
        Annotated[
            'BGPSessionTemplateType', strawberry.lazy('netbox_routing.graphql.types')
        ],
        Annotated['BGPRouterType', strawberry.lazy('netbox_routing.graphql.types')],
        Annotated['BGPScopeType', strawberry.lazy('netbox_routing.graphql.types')],
        Annotated[
            'BGPAddressFamilyType', strawberry.lazy('netbox_routing.graphql.types')
        ],
        Annotated['BGPPeerType', strawberry.lazy('netbox_routing.graphql.types')],
        Annotated[
            'BGPPeerAddressFamilyType', strawberry.lazy('netbox_routing.graphql.types')
        ],
    ]
    key: str
    value: (
        Annotated["ASNType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.BGPPeerTemplate, fields='__all__', filters=BGPPeerTemplateFilter
)
class BGPPeerTemplateType(PrimaryObjectType):

    name: str
    remote_as: (
        Annotated["ASNType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821
    enabled: bool | None
    tenant: (
        Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.BGPPolicyTemplate, fields='__all__', filters=BGPPolicyTemplateFilter
)
class BGPPolicyTemplateType(PrimaryObjectType):

    name: str
    parents: (
        List[
            Annotated[
                "BGPPolicyTemplateType", strawberry.lazy('netbox_routing.graphql.types')
            ]
        ]
        | None
    )
    enabled: bool | None
    prefixlist_in: (
        Annotated["PrefixListType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    prefixlist_out: (
        Annotated["PrefixListType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    routemap_in: (
        Annotated["RouteMapType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    routemap_out: (
        Annotated["RouteMapType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    tenant: (
        Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.BGPSessionTemplate, fields='__all__', filters=BGPSessionTemplateFilter
)
class BGPSessionTemplateType(BGPSettingsMixin, PrimaryObjectType):

    name: str
    parent: (
        Annotated[
            "BGPSessionTemplateType", strawberry.lazy('netbox_routing.graphql.types')
        ]
        | None
    )
    enabled: bool | None
    remote_as: (
        Annotated["ASNType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821
    local_as: (
        Annotated["ASNType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821
    bfd: (
        Annotated["BFDProfileType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    ttl: int | None
    password: str | None
    tenant: Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None


@strawberry_django.type(
    models.BGPRouter,
    fields='__all__',
    exclude=['assigned_object_type', 'assigned_object_id'],
    filters=BGPRouterFilter,
    select_related=[
        'asn',
    ],
)
class BGPRouterType(BGPSettingsMixin, PrimaryObjectType):

    name: str | None
    assigned_object_type: (
        Annotated["ContentTypeType", strawberry.lazy('netbox.graphql.types')] | None
    )  # noqa: F821
    assigned_object: Union[
        Annotated['RegionType', strawberry.lazy('dcim.graphql.types')],
        Annotated['SiteType', strawberry.lazy('dcim.graphql.types')],
        Annotated['SiteGroupType', strawberry.lazy('dcim.graphql.types')],
        Annotated['LocationType', strawberry.lazy('dcim.graphql.types')],
        Annotated['DeviceType', strawberry.lazy('dcim.graphql.types')],
        Annotated['ClusterType', strawberry.lazy('virtualization.graphql.types')],
        Annotated[
            'VirtualMachineType', strawberry.lazy('virtualization.graphql.types')
        ],
    ]
    asn: Annotated["ASNType", strawberry.lazy('ipam.graphql.types')]
    peer_templates: (
        List[
            Annotated[
                'BGPPeerTemplateType', strawberry.lazy('netbox_routing.graphql.types')
            ]
        ]
        | None
    )
    policy_templates: (
        List[
            Annotated[
                'BGPPolicyTemplateType', strawberry.lazy('netbox_routing.graphql.types')
            ]
        ]
        | None
    )
    session_templates: (
        List[
            Annotated[
                'BGPSessionTemplateType',
                strawberry.lazy('netbox_routing.graphql.types'),
            ]
        ]
        | None
    )
    tenant: (
        Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.BGPScope,
    fields='__all__',
    filters=BGPScopeFilter,
    select_related=[
        'router',
        'vrf',
    ],
)
class BGPScopeType(BGPSettingsMixin, PrimaryObjectType):

    router: Annotated["BGPRouterType", strawberry.lazy('netbox_routing.graphql.types')]
    vrf: (
        Annotated["VRFType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821
    tenant: (
        Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.BGPAddressFamily,
    fields='__all__',
    filters=BGPAddressFamilyFilter,
    select_related=[
        'scope',
    ],
)
class BGPAddressFamilyType(BGPSettingsMixin, PrimaryObjectType):

    scope: Annotated["BGPScopeType", strawberry.lazy('netbox_routing.graphql.types')]
    address_family: str | None
    tenant: (
        Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.BGPPeer,
    fields='__all__',
    filters=BGPPeerFilter,
    select_related=['scope', 'peer', 'remote_as'],
)
class BGPPeerType(BGPSettingsMixin, PrimaryObjectType):

    name: str | None
    enabled: bool | None
    status: str | None
    scope: (
        Annotated["BGPScopeType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    peer_group: (
        Annotated[
            "BGPPeerTemplateType", strawberry.lazy('netbox_routing.graphql.types')
        ]
        | None
    )
    peer_session: (
        Annotated[
            "BGPSessionTemplateType", strawberry.lazy('netbox_routing.graphql.types')
        ]
        | None
    )
    peer: Annotated[
        "IPAddressType", strawberry.lazy('ipam.graphql.types')
    ]  # noqa: F821
    source: (
        Annotated["IPAddressType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821
    remote_as: (
        Annotated["ASNType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821
    local_as: (
        Annotated["ASNType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821
    bfd: (
        Annotated["BFDProfileType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    ttl: int | None
    password: str | None
    address_families: (
        List[
            Annotated[
                'BGPPeerAddressFamilyType',
                strawberry.lazy('netbox_routing.graphql.types'),
            ]
        ]
        | None
    )
    tenant: (
        Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.BGPPeerAddressFamily,
    fields='__all__',
    exclude=['assigned_object_type', 'assigned_object_id'],
    filters=BGPPeerAddressFamilyFilter,
    select_related=[
        'assigned_object_type',
        'address_family',
    ],
)
class BGPPeerAddressFamilyType(BGPSettingsMixin, PrimaryObjectType):

    assigned_object_type: (
        Annotated["ContentTypeType", strawberry.lazy('netbox.graphql.types')] | None
    )  # noqa: F821
    assigned_object: (
        Union[
            Annotated["BGPPeerType", strawberry.lazy('netbox_routing.graphql.types')],
            Annotated[
                "BGPPeerTemplateType", strawberry.lazy('netbox_routing.graphql.types')
            ],
        ]
        | None
    )
    peer_policy: (
        Annotated[
            "BGPPolicyTemplateType", strawberry.lazy('netbox_routing.graphql.types')
        ]
        | None
    )
    enabled: bool | None
    prefixlist_in: (
        Annotated['PrefixListType', strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    prefixlist_out: (
        Annotated['PrefixListType', strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    routemap_in: (
        Annotated['RouteMapType', strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    routemap_out: (
        Annotated['RouteMapType', strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    address_family: (
        Annotated[
            'BGPAddressFamilyType',
            strawberry.lazy('netbox_routing.graphql.types'),
        ]
        | None
    )
    tenant: (
        Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.BFDProfile,
    fields='__all__',
    filters=BFDProfileFilter,
)
class BFDProfileType(PrimaryObjectType):
    name: str
    min_tx_int: int
    min_rx_int: int
    multiplier: int
    hold: int | None
    tenant: Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None



from django.test import TestCase

from ipam.models import Role
from netbox_routing.filtersets.community import *
from netbox_routing.models.community import *

__all__ = (
    'CommunityTestCase',
    'CommunityListTestCase',
    'CommunityListEntryTestCase',
)

from tenancy.models import Tenant


class CommunityTestCase(TestCase):
    queryset = Community.objects.all()
    filterset = CommunityFilterSet

    @classmethod
    def setUpTestData(cls):
        cls.tenants = (
            Tenant(name='Test Tenant 1', slug='test-tenant-1'),
            Tenant(name='Test Tenant 2', slug='test-tenant-2'),
        )
        Tenant.objects.bulk_create(cls.tenants)

        cls.roles = (
            Role(name='Test Role 1', slug='test-role-1'),
            Role(name='Test Role 2', slug='test-role-2'),
        )
        Role.objects.bulk_create(cls.roles)

        cls.communities = [
            Community(
                community='64512',
                status='active',
                role=cls.roles[0],
                tenant=cls.tenants[0],
            ),
            Community(
                community='64513',
                status='reserved',
                role=cls.roles[0],
                tenant=cls.tenants[1],
            ),
            Community(
                community='64514',
                status='active',
                role=cls.roles[1],
                tenant=cls.tenants[1],
            ),
            Community(
                community='64515',
                status='reserved',
                role=cls.roles[1],
                tenant=cls.tenants[0],
            ),
        ]
        Community.objects.bulk_create(cls.communities)

    def test_q(self):
        params = {'q': '64512'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)

    def test_status(self):
        params = {'status': 'active'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
        params = {'status': 'reserved'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
        params = {'status': 'deprecated'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)

    def test_role(self):
        params = {
            'role_id': [
                self.roles[1].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'role': [
                self.roles[1].name,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_tenant(self):
        params = {
            'tenant_id': [
                self.tenants[1].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'tenant': [
                self.tenants[1].slug,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class CommunityListTestCase(TestCase):
    queryset = CommunityList.objects.all()
    filterset = CommunityListFilterSet

    @classmethod
    def setUpTestData(cls):
        cls.tenants = (
            Tenant(name='Test Tenant 1', slug='test-tenant-1'),
            Tenant(name='Test Tenant 2', slug='test-tenant-2'),
        )
        Tenant.objects.bulk_create(cls.tenants)

        cls.community_lists = [
            CommunityList(
                name='Test Community List 1-1',
                tenant=cls.tenants[0],
            ),
            CommunityList(
                name='Test Community List 1-2',
                tenant=cls.tenants[1],
            ),
            CommunityList(name='Test Community List 2-1', tenant=cls.tenants[1]),
            CommunityList(name='Test Community List 2-2', tenant=cls.tenants[0]),
        ]
        CommunityList.objects.bulk_create(cls.community_lists)

    def test_q(self):
        params = {'q': 'Test Community List 1-'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_tenant(self):
        params = {
            'tenant_id': [
                self.tenants[1].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'tenant': [
                self.tenants[1].slug,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class CommunityListEntryTestCase(TestCase):
    queryset = CommunityListEntry.objects.all()
    filterset = CommunityListEntryFilterSet

    @classmethod
    def setUpTestData(cls):
        cls.tenants = (Tenant(name='Test Tenant 1', slug='test-tenant-1'),)
        Tenant.objects.bulk_create(cls.tenants)

        cls.roles = (Role(name='Test Role 1', slug='test-role-1'),)
        Role.objects.bulk_create(cls.roles)

        cls.communities = [
            Community(
                community='64512',
                status='active',
                role=cls.roles[0],
                tenant=cls.tenants[0],
            ),
            Community(
                community='64513',
                status='reserved',
                role=cls.roles[0],
                tenant=cls.tenants[0],
            ),
        ]
        Community.objects.bulk_create(cls.communities)

        cls.community_lists = [
            CommunityList(
                name='Test Community List 1',
                tenant=cls.tenants[0],
            ),
            CommunityList(
                name='Test Community List 2',
                tenant=cls.tenants[0],
            ),
        ]
        CommunityList.objects.bulk_create(cls.community_lists)

        cls.community_list_entries = [
            CommunityListEntry(
                community_list=cls.community_lists[0],
                community=cls.communities[0],
                action='permit',
            ),
            CommunityListEntry(
                community_list=cls.community_lists[0],
                community=cls.communities[1],
                action='permit',
            ),
            CommunityListEntry(
                community_list=cls.community_lists[1],
                community=cls.communities[0],
                action='permit',
            ),
            CommunityListEntry(
                community_list=cls.community_lists[1],
                community=cls.communities[1],
                action='permit',
            ),
        ]
        CommunityListEntry.objects.bulk_create(cls.community_list_entries)

    def test_q(self):
        params = {'q': 'Test Community List 1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_community(self):
        params = {
            'community_id': [
                self.communities[1].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'community': [
                self.communities[1].community,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_community_list(self):
        params = {
            'community_list_id': [
                self.community_lists[1].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'community_list': [
                self.community_lists[1].name,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)



from django import forms
from django.utils.translation import gettext as _

from ipam.models import Role
from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.choices import CommunityStatusChoices, ActionChoices
from tenancy.forms import TenancyFilterForm
from utilities.forms import add_blank_choice
from utilities.forms.fields import TagFilterField, DynamicModelMultipleChoiceField
from utilities.forms.rendering import FieldSet

from netbox_routing.models.community import *

__all__ = (
    'CommunityListFilterForm',
    'CommunityListEntryFilterForm',
    'CommunityFilterForm',
)


class CommunityListFilterForm(TenancyFilterForm, NetBoxModelFilterSetForm):
    model = CommunityList
    fieldsets = (
        FieldSet(
            'q',
            'filter_id',
            'tag',
        ),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    tag = TagFilterField(model)


class CommunityFilterForm(TenancyFilterForm, NetBoxModelFilterSetForm):
    model = Community
    fieldsets = (
        FieldSet('q', 'filter_id', 'tag'),
        FieldSet('community', 'status', 'role_id', name=_('Community')),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )
    status = forms.ChoiceField(
        required=False,
        choices=add_blank_choice(CommunityStatusChoices),
        label=_('Status'),
    )
    role = DynamicModelMultipleChoiceField(
        queryset=Role.objects.all(),
        required=False,
        selector=True,
        label=_('Role'),
    )
    tag = TagFilterField(model)


class CommunityListEntryFilterForm(NetBoxModelFilterSetForm):
    model = CommunityListEntry
    fieldsets = (
        FieldSet('q', 'filter_id', 'tag'),
        FieldSet('community_list', 'action', 'community', name=_('Community')),
    )
    action = forms.ChoiceField(
        required=False,
        choices=add_blank_choice(ActionChoices),
        label=_('Action'),
    )
    community = DynamicModelMultipleChoiceField(
        queryset=Community.objects.all(),
        required=False,
        selector=True,
        label=_('Community'),
    )
    community_list = DynamicModelMultipleChoiceField(
        queryset=CommunityList.objects.all(),
        required=False,
        selector=True,
        label=_('Community List'),
    )
    tag = TagFilterField(model)



# Generated by Django 5.2.5 on 2026-02-14 17:47

import django.db.models.deletion
import ipam.fields
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


def forward_migrate(apps, schema_editor):
    from django.contrib.contenttypes.models import ContentType

    PrefixListEntry = apps.get_model('netbox_routing', 'PrefixListEntry')
    CustomPrefix = apps.get_model('netbox_routing', 'CustomPrefix')
    Prefix = apps.get_model('ipam', 'Prefix')

    pfx_ct = ContentType.objects.get_for_model(Prefix)
    cpfx_ct = ContentType.objects.get_for_model(CustomPrefix)

    for ple in PrefixListEntry.objects.all():
        if ple.prefix:
            try:
                prefix = Prefix.objects.get(prefix=ple.prefix)
                ple.assigned_prefix_type_id = pfx_ct.pk
                ple.assigned_prefix_id = prefix.pk
                ple.save()
            except (Prefix.DoesNotExist, Prefix.MultipleObjectsReturned):
                try:
                    prefix = CustomPrefix.objects.get(prefix=ple.prefix)
                except CustomPrefix.DoesNotExist:
                    prefix = CustomPrefix.objects.create(prefix=ple.prefix)
                ple.assigned_prefix_type_id = cpfx_ct.pk
                ple.assigned_prefix_id = prefix.pk
                ple.save()


def reverse_migrate(apps, schema_editor):
    PrefixListEntry = apps.get_model('netbox_routing', 'PrefixListEntry')
    Prefix = apps.get_model('ipam', 'Prefix')
    CustomPrefix = apps.get_model('netbox_routing', 'CustomPrefix')
    ContentType = apps.get_model('contenttypes', 'ContentType')

    pfx_ct = ContentType.objects.get(app_label='ipam', model='prefix')
    cpfx_ct = ContentType.objects.get(app_label='netbox_routing', model='customprefix')

    for ple in PrefixListEntry.objects.all():
        if ple.assigned_prefix_type == pfx_ct:
            prefix = Prefix.objects.get(pk=ple.assigned_prefix_id)
            ple.prefix = prefix.prefix
            ple.save()
        elif ple.assigned_prefix_type == cpfx_ct:
            prefix = CustomPrefix.objects.get(pk=ple.assigned_prefix_id)
            ple.prefix = prefix.prefix
            ple.save()


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('extras', '0134_owner'),
        ('netbox_routing', '0024_bfdprofile'),
        ('users', '0015_owner'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomPrefix',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('prefix', ipam.fields.IPNetworkField()),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'ordering': ['prefix'],
                'constraints': [
                    models.UniqueConstraint(
                        fields=('prefix',),
                        name='netbox_routing_customprefix_unique_prefix',
                        violation_error_message='Prefix must be unique.',
                    )
                ],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.AddField(
            model_name='prefixlistentry',
            name='assigned_prefix_id',
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='prefixlistentry',
            name='assigned_prefix_type',
            field=models.ForeignKey(
                blank=True,
                limit_choices_to=models.Q(
                    models.Q(
                        models.Q(('app_label', 'ipam'), ('model', 'prefix')),
                        models.Q(
                            ('app_label', 'netbox_routing'),
                            ('model', 'customprefix'),
                        ),
                        _connector='OR',
                    )
                ),
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name='+',
                to='contenttypes.contenttype',
            ),
        ),
        migrations.AlterField(
            model_name='prefixlistentry',
            name='prefix',
            field=ipam.fields.IPNetworkField(blank=True, null=True),
        ),
        migrations.RunPython(forward_migrate, reverse_code=reverse_migrate),
        migrations.RemoveField(
            model_name='prefixlistentry',
            name='prefix',
        ),
    ]



from django.test import TestCase

from netbox_routing.models import *

__all__ = (
    'OSPFInstanceTestCase',
    'OSPFAreaTestCase',
    'OSPFInterfaceTestCase',
)


class OSPFInstanceTestCase(TestCase):
    pass


class OSPFAreaTestCase(TestCase):
    pass


class OSPFInterfaceTestCase(TestCase):
    pass



from django.test import TestCase

from ipam.models import Prefix
from netbox_routing.choices import ActionChoices

from netbox_routing.filtersets.objects import *
from netbox_routing.models.objects import *

__all__ = (
    'ASPathTestCase',
    'ASPathEntryTestCase',
    'PrefixListTestCase',
    'PrefixListEntryTestCase',
    'RouteMapTestCase',
    'RouteMapEntryTestCase',
)


class ASPathTestCase(TestCase):
    queryset = ASPath.objects.all()
    filterset = ASPathFilterSet

    @classmethod
    def setUpTestData(cls):

        cls.aspath = [
            ASPath(
                name='AS Path List 1-1',
            ),
            ASPath(
                name='AS Path List 1-2',
            ),
            ASPath(
                name='AS Path List 2-1',
            ),
            ASPath(
                name='AS Path List 2-2',
            ),
        ]
        ASPath.objects.bulk_create(cls.aspath)

    def test_q(self):
        params = {'q': 'AS Path List 1-'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class ASPathEntryTestCase(TestCase):
    queryset = ASPathEntry.objects.all()
    filterset = ASPathEntryFilterSet

    @classmethod
    def setUpTestData(cls):
        cls.aspath = (
            ASPath(
                name='AS Path List 1',
            ),
            ASPath(
                name='AS Path List 2',
            ),
        )
        ASPath.objects.bulk_create(cls.aspath)

        cls.aspath_entry = [
            ASPathEntry(
                aspath=cls.aspath[0],
                action=ActionChoices.PERMIT,
                sequence=1,
                pattern='^1234$',
            ),
            ASPathEntry(
                aspath=cls.aspath[0],
                action=ActionChoices.PERMIT,
                sequence=2,
                pattern='^5678$',
            ),
            ASPathEntry(
                aspath=cls.aspath[1],
                action=ActionChoices.PERMIT,
                sequence=1,
                pattern='^1234$',
            ),
            ASPathEntry(
                aspath=cls.aspath[1],
                action=ActionChoices.PERMIT,
                sequence=2,
                pattern='^5678',
            ),
        ]
        ASPathEntry.objects.bulk_create(cls.aspath_entry)

    def test_q(self):
        params = {'q': '1234'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_aspath(self):
        params = {
            'aspath_id': [
                self.aspath[1].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'aspath': [
                self.aspath[1].name,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class PrefixListTestCase(TestCase):
    queryset = PrefixList.objects.all()
    filterset = PrefixListFilterSet

    @classmethod
    def setUpTestData(cls):

        cls.prefix_list = [
            PrefixList(
                name='Prefix List 1-1',
            ),
            PrefixList(
                name='Prefix List 1-2',
            ),
            PrefixList(
                name='Prefix List 2-1',
            ),
            PrefixList(
                name='Prefix List 2-2',
            ),
        ]
        PrefixList.objects.bulk_create(cls.prefix_list)

    def test_q(self):
        params = {'q': 'Prefix List 1-'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class PrefixListEntryTestCase(TestCase):
    queryset = PrefixListEntry.objects.all()
    filterset = PrefixListEntryFilterSet

    @classmethod
    def setUpTestData(cls):
        cls.prefix_list = (
            PrefixList(
                name='Prefix List 1',
            ),
            PrefixList(
                name='Prefix List 2',
            ),
        )
        PrefixList.objects.bulk_create(cls.prefix_list)

        cls.prefixes = (Prefix(prefix='10.0.0.0/24'),)
        Prefix.objects.bulk_create(cls.prefixes)

        cls.custom_prefixes = (CustomPrefix(prefix='10.0.1.0/24'),)
        CustomPrefix.objects.bulk_create(cls.custom_prefixes)

        cls.prefix_list_entry = [
            PrefixListEntry(
                prefix_list=cls.prefix_list[0],
                action=ActionChoices.PERMIT,
                sequence=1,
                assigned_prefix=cls.prefixes[0],
            ),
            PrefixListEntry(
                prefix_list=cls.prefix_list[0],
                action=ActionChoices.PERMIT,
                sequence=2,
                assigned_prefix=cls.custom_prefixes[0],
            ),
            PrefixListEntry(
                prefix_list=cls.prefix_list[1],
                action=ActionChoices.PERMIT,
                sequence=1,
                assigned_prefix=cls.prefixes[0],
            ),
            PrefixListEntry(
                prefix_list=cls.prefix_list[1],
                action=ActionChoices.PERMIT,
                sequence=2,
                assigned_prefix=cls.custom_prefixes[0],
            ),
        ]
        PrefixListEntry.objects.bulk_create(cls.prefix_list_entry)

    def test_q(self):
        params = {'q': 'Prefix List 1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_prefix_list(self):
        params = {
            'prefix_list_id': [
                self.prefix_list[1].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'prefix_list': [
                self.prefix_list[1].name,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_prefix(self):
        params = {
            'prefix_id': [
                self.prefixes[0].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'prefix': [
                self.prefixes[0].prefix,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_custom_prefix(self):
        params = {
            'custom_prefix_id': [
                self.custom_prefixes[0].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'custom_prefix': [
                self.custom_prefixes[0].prefix,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class RouteMapTestCase(TestCase):
    queryset = RouteMap.objects.all()
    filterset = RouteMapFilterSet

    @classmethod
    def setUpTestData(cls):

        cls.prefix_list = [
            RouteMap(
                name='Route Map 1-1',
            ),
            RouteMap(
                name='Route Map 1-2',
            ),
            RouteMap(
                name='Route Map 2-1',
            ),
            RouteMap(
                name='Route Map 2-2',
            ),
        ]
        RouteMap.objects.bulk_create(cls.prefix_list)

    def test_q(self):
        params = {'q': 'Route Map 1-'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class RouteMapEntryTestCase(TestCase):
    queryset = RouteMapEntry.objects.all()
    filterset = RouteMapEntryFilterSet

    @classmethod
    def setUpTestData(cls):
        cls.route_map = (
            RouteMap(
                name='Route Map 1',
            ),
            RouteMap(
                name='Route Map 2',
            ),
        )
        RouteMap.objects.bulk_create(cls.route_map)

        cls.route_map_entry = [
            RouteMapEntry(
                route_map=cls.route_map[0],
                action=ActionChoices.PERMIT,
                sequence=1,
                match={'tag': 1234},
            ),
            RouteMapEntry(
                route_map=cls.route_map[0],
                action=ActionChoices.PERMIT,
                sequence=2,
                match={'tag': 5678},
            ),
            RouteMapEntry(
                route_map=cls.route_map[1],
                action=ActionChoices.PERMIT,
                sequence=1,
                match={'tag': 9012},
            ),
            RouteMapEntry(
                route_map=cls.route_map[1],
                action=ActionChoices.PERMIT,
                sequence=2,
                match={'tag': 3456},
            ),
        ]
        RouteMapEntry.objects.bulk_create(cls.route_map_entry)

    def test_q(self):
        params = {'q': 'Route Map 1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_match(self):
        params = {'q': '34'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_prefix_list(self):
        params = {
            'route_map_id': [
                self.route_map[1].pk,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {
            'route_map': [
                self.route_map[1].name,
            ]
        }
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)



from django.db.models import Q

__all__ = (
    'BGPROUTER_ASSIGNMENT_MODEL',
    'BGPSETTING_ASSIGNMENT_MODELS',
    'BGPAF_ASSIGNMENT_MODELS',
    'BGPPEER_ASSIGNMENT_MODELS',
    'BGPPEERAF_ASSIGNMENT_MODELS',
)


BGPROUTER_ASSIGNMENT_MODEL = Q(
    Q(app_label='dcim', model='region')
    | Q(app_label='dcim', model='site')
    | Q(app_label='dcim', model='site_group')
    | Q(app_label='dcim', model='location')
    | Q(app_label='dcim', model='device')
    | Q(app_label='virtualization', model='cluster')
    | Q(app_label='virtualization', model='virtual_machine')
)

BGPSETTING_ASSIGNMENT_MODELS = Q(
    Q(app_label='netbox_routing', model='bgprouter')
    | Q(app_label='netbox_routing', model='bgpscope')
    | Q(app_label='netbox_routing', model='bgpaddressfamily')
    | Q(app_label='netbox_routing', model='bgppeer')
    | Q(app_label='netbox_routing', model='bgppeeraddressfamily')
)

BGPAF_ASSIGNMENT_MODELS = Q(
    Q(app_label='netbox_routing', model='bgprouter')
    | Q(app_label='netbox_routing', model='bgpscope')
)

BGPPEER_ASSIGNMENT_MODELS = Q(
    Q(app_label='netbox_routing', model='bgprouter')
    | Q(app_label='netbox_routing', model='bgpscope')
)

BGPPEERAF_ASSIGNMENT_MODELS = Q(
    Q(app_label='netbox_routing', model='bgppeer')
    | Q(app_label='netbox_routing', model='bgppeertemplate')
)



# Generated by Django 5.0.8 on 2024-09-27 13:00

import django.db.models.deletion
import netbox_routing.fields.ip
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0009_alter_staticroute_metric_alter_staticroute_permanent'),
    ]

    operations = [
        migrations.CreateModel(
            name='EIGRPAddressFamily',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('family', models.PositiveSmallIntegerField()),
                ('rid', netbox_routing.fields.ip.IPAddressField(blank=True, null=True)),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'vrf',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='eigrp_address_families',
                        to='ipam.vrf',
                    ),
                ),
            ],
            options={
                'verbose_name': 'EIGRP Address Family',
            },
        ),
        migrations.CreateModel(
            name='EIGRPRouter',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('mode', models.CharField(max_length=10)),
                ('name', models.CharField(blank=True, max_length=100, null=True)),
                ('pid', models.PositiveIntegerField(blank=True, null=True)),
                ('rid', netbox_routing.fields.ip.IPAddressField(blank=True, null=True)),
                (
                    'device',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='eigrp',
                        to='dcim.device',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'verbose_name': 'EIGRP Router',
            },
        ),
        migrations.CreateModel(
            name='EIGRPNetwork',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                (
                    'address_family',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to='netbox_routing.eigrpaddressfamily',
                    ),
                ),
                (
                    'network',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='eigrp',
                        to='ipam.prefix',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'router',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='netbox_routing.eigrprouter',
                    ),
                ),
            ],
            options={
                'verbose_name': 'EIGRP Network',
            },
        ),
        migrations.CreateModel(
            name='EIGRPInterface',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('passive', models.BooleanField()),
                ('bfd', models.BooleanField()),
                (
                    'authentication',
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                ('passphrase', models.CharField(blank=True, max_length=200, null=True)),
                (
                    'address_family',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to='netbox_routing.eigrpaddressfamily',
                    ),
                ),
                (
                    'interface',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='eigrp',
                        to='dcim.interface',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'router',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='netbox_routing.eigrprouter',
                    ),
                ),
            ],
            options={
                'verbose_name': 'EIGRP Interface',
            },
        ),
        migrations.AddField(
            model_name='eigrpaddressfamily',
            name='router',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to='netbox_routing.eigrprouter',
            ),
        ),
        migrations.AddConstraint(
            model_name='eigrpnetwork',
            constraint=models.UniqueConstraint(
                fields=('router', 'address_family', 'network'),
                name='netbox_routing_eigrpnetwork_unique_network',
            ),
        ),
        migrations.AddConstraint(
            model_name='eigrpinterface',
            constraint=models.UniqueConstraint(
                fields=('router', 'address_family', 'interface'),
                name='netbox_routing_eigrpinterface_unique_interface',
            ),
        ),
        migrations.AddConstraint(
            model_name='eigrpaddressfamily',
            constraint=models.UniqueConstraint(
                fields=('router', 'vrf', 'family'),
                name='netbox_routing_eigrpaddressfamily_unique_af',
            ),
        ),
    ]



from django.utils.translation import gettext as _

from netbox.forms import PrimaryModelBulkEditForm
from utilities.forms.fields import DynamicModelChoiceField
from utilities.forms.rendering import FieldSet

from netbox_routing.models.objects import *

__all__ = (
    'PrefixListBulkEditForm',
    'PrefixListEntryBulkEditForm',
    'RouteMapBulkEditForm',
    'RouteMapEntryBulkEditForm',
    'ASPathBulkEditForm',
    'ASPathEntryBulkEditForm',
)


class PrefixListBulkEditForm(PrimaryModelBulkEditForm):

    model = PrefixList
    fieldsets = (
        FieldSet(
            'description',
        ),
    )
    nullable_fields = ('description',)


class PrefixListEntryBulkEditForm(PrimaryModelBulkEditForm):
    prefix_list = DynamicModelChoiceField(
        queryset=PrefixList.objects.all(),
        label=_('Prefix List'),
        required=False,
        selector=True,
    )

    model = PrefixListEntry
    fieldsets = (
        FieldSet(
            'prefix_list',
            'description',
        ),
    )
    nullable_fields = ('description',)


class RouteMapBulkEditForm(PrimaryModelBulkEditForm):

    model = RouteMap
    fieldsets = (
        FieldSet(
            'description',
        ),
    )
    nullable_fields = ('description',)


class RouteMapEntryBulkEditForm(PrimaryModelBulkEditForm):
    route_map = DynamicModelChoiceField(
        queryset=RouteMap.objects.all(),
        label=_('Route Map'),
        required=False,
        selector=True,
    )

    model = RouteMapEntry
    fieldsets = (
        FieldSet(
            'route_map',
            'description',
        ),
    )
    nullable_fields = ('description',)


class ASPathBulkEditForm(PrimaryModelBulkEditForm):

    model = ASPath
    fieldsets = (
        FieldSet(
            'description',
        ),
    )
    nullable_fields = ('description',)


class ASPathEntryBulkEditForm(PrimaryModelBulkEditForm):
    aspath = DynamicModelChoiceField(
        queryset=ASPath.objects.all(),
        label=_('AS Path'),
        required=False,
        selector=True,
    )

    model = ASPathEntry
    fieldsets = (
        FieldSet(
            'aspath',
            'description',
        ),
    )
    nullable_fields = ('description',)



from django.core.exceptions import ValidationError
from django.db import models
from netaddr import AddrFormatError, IPAddress

from ipam import lookups
from ipam.formfields import IPAddressFormField


class IPAddressField(models.Field):

    def python_type(self):
        return IPAddress

    def from_db_value(self, value, expression, connection):
        return self.to_python(value)

    def to_python(self, value):
        if value is None:
            return None
        elif not value:
            return value
        try:
            # Always return a netaddr.IPNetwork object. (netaddr.IPAddress does not provide a mask.)
            return IPAddress(value)
        except AddrFormatError:
            raise ValidationError("Invalid IP address format: {}".format(value))
        except (TypeError, ValueError) as e:
            raise ValidationError(e)

    def get_prep_value(self, value):
        if value is None:
            return None
        if isinstance(value, list):
            return [str(self.to_python(v)) for v in value]
        return str(self.to_python(value))

    def form_class(self):
        return IPAddressFormField

    def formfield(self, **kwargs):
        defaults = {'form_class': self.form_class()}
        defaults.update(kwargs)
        return super().formfield(**defaults)


IPAddressField.register_lookup(lookups.IExact)
IPAddressField.register_lookup(lookups.EndsWith)
IPAddressField.register_lookup(lookups.IEndsWith)
IPAddressField.register_lookup(lookups.StartsWith)
IPAddressField.register_lookup(lookups.IStartsWith)
IPAddressField.register_lookup(lookups.Regex)
IPAddressField.register_lookup(lookups.IRegex)
IPAddressField.register_lookup(lookups.NetContained)
IPAddressField.register_lookup(lookups.NetContainedOrEqual)
IPAddressField.register_lookup(lookups.NetFamily)



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable
from netbox_routing.models import StaticRoute


class StaticRouteTable(NetBoxTable):
    devices = tables.ManyToManyColumn(
        verbose_name=_('Devices'),
        linkify_item=True,
    )
    vrf = tables.Column(
        verbose_name=_('VRF'),
        linkify=True,
    )

    class Meta(NetBoxTable.Meta):
        model = StaticRoute
        fields = (
            'pk',
            'id',
            'devices',
            'vrf',
            'prefix',
            'next_hop',
            'interface_next_hop',
            'name',
            'metric',
            'permanent',
            'tag',
            'description',
            'comments',
        )
        default_columns = (
            'pk',
            'id',
            'devices',
            'vrf',
            'prefix',
            'next_hop',
            'interface_next_hop',
            'name',
        )



from netbox.search import SearchIndex, register_search, ObjectFieldValue
from netbox_routing.models.objects import *


@register_search
class ASPathIndex(SearchIndex):
    model = ASPath
    fields = (
        ('name', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('name',)


@register_search
class ASPathEntryIndex(SearchIndex):
    model = ASPathEntry
    fields = (
        ('aspath', 100),
        ('pattern', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'aspath',
        'pattern',
    )


@register_search
class PrefixListIndex(SearchIndex):
    model = PrefixList
    fields = (
        ('name', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('name',)


@register_search
class PrefixListEntryIndex(SearchIndex):
    model = PrefixListEntry
    fields = (
        ('prefix_list', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'prefix_list',
        'assigned_prefix',
    )

    @classmethod
    def to_cache(cls, instance, custom_fields=None):
        values = super().to_cache(instance, custom_fields)
        if hasattr(instance, 'assigned_prefix'):
            type_ = cls.get_field_type(instance.assigned_prefix, 'prefix')
            value = cls.get_field_value(instance.assigned_prefix, 'prefix')
            values.append(ObjectFieldValue('assigned_prefix', type_, 100, value))

        return values


@register_search
class RouteMapIndex(SearchIndex):
    model = RouteMap
    fields = (
        ('name', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('name',)


@register_search
class RouteMapEntryIndex(SearchIndex):
    model = RouteMapEntry
    fields = (
        ('route_map', 100),
        ('match', 200),
        ('set', 300),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('route_map',)



from rest_framework import serializers

from dcim.api.serializers_.devices import DeviceSerializer
from ipam.api.serializers_.vrfs import VRFSerializer
from netbox.api.serializers import NetBoxModelSerializer

from netbox_routing.api.field_serializers import IPAddressField
from netbox_routing.models import StaticRoute

__all__ = 'StaticRouteSerializer'


class StaticRouteSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:staticroute-detail'
    )
    devices = DeviceSerializer(many=True, nested=True, required=False, allow_null=True)
    vrf = VRFSerializer(nested=True, required=False, allow_null=True)
    next_hop = IPAddressField()

    class Meta:
        model = StaticRoute
        fields = (
            'url',
            'id',
            'display',
            'devices',
            'vrf',
            'prefix',
            'next_hop',
            'name',
            'metric',
            'tag',
            'permanent',
            'description',
            'comments',
            'custom_fields',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
            'prefix',
            'next_hop',
            'description',
        )

    def create(self, validated_data):
        devices = validated_data.pop('devices', None)
        instance = super(StaticRouteSerializer, self).create(validated_data)

        return self._update_devices(instance, devices)

    def update(self, instance, validated_data):
        devices = validated_data.pop('devices', None)
        instance = super(StaticRouteSerializer, self).update(instance, validated_data)

        return self._update_devices(instance, devices)

    def _update_devices(self, instance: StaticRoute, devices: object) -> StaticRoute:
        if devices:
            instance.devices.set(devices)
        elif devices is not None:
            instance.devices.clear()

        return instance



from django import forms
from django.core.exceptions import ValidationError
from django.forms import ChoiceField
from django.utils.translation import gettext as _

from dcim.models import Interface, Device
from ipam.choices import IPAddressFamilyChoices
from ipam.models import VRF, Prefix
from netbox.forms import NetBoxModelForm
from netbox_routing.choices.eigrp import EIGRPRouterChoices
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES, get_field_value
from utilities.forms.fields import DynamicModelChoiceField, CommentField
from utilities.forms.widgets import HTMXSelect

from netbox_routing.models import *

__all__ = (
    'EIGRPRouterForm',
    'EIGRPAddressFamilyForm',
    'EIGRPNetworkForm',
    'EIGRPInterfaceForm',
)


class EIGRPRouterForm(NetBoxModelForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=True,
        selector=True,
        label=_('Device'),
    )
    comments = CommentField()

    class Meta:
        model = EIGRPRouter
        fields = (
            'device',
            'mode',
            'name',
            'pid',
            'rid',
            'description',
            'comments',
        )
        widgets = {
            'mode': HTMXSelect(),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        mode = get_field_value(self, 'mode')
        if mode == EIGRPRouterChoices.NAMED:
            del self.fields['pid']
        elif mode == EIGRPRouterChoices.CLASSIC:
            del self.fields['name']


class EIGRPAddressFamilyForm(NetBoxModelForm):
    router = DynamicModelChoiceField(
        queryset=EIGRPRouter.objects.all(),
        required=True,
        selector=True,
        label=_('Router'),
    )
    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )
    family = ChoiceField(
        required=True,
        choices=IPAddressFamilyChoices,
        label=_('Family'),
    )
    comments = CommentField()

    class Meta:
        model = EIGRPAddressFamily
        fields = (
            'router',
            'vrf',
            'family',
            'rid',
            'description',
            'comments',
        )


class EIGRPNetworkForm(NetBoxModelForm):
    router = DynamicModelChoiceField(
        queryset=EIGRPRouter.objects.all(),
        required=True,
        selector=True,
        label=_('Router'),
    )
    address_family = DynamicModelChoiceField(
        queryset=EIGRPAddressFamily.objects.all(),
        required=False,
        selector=True,
        label=_('Address Family'),
    )
    network = DynamicModelChoiceField(
        queryset=Prefix.objects.all(),
        required=True,
        selector=True,
        label=_('Prefix'),
    )
    comments = CommentField()

    class Meta:
        model = EIGRPNetwork
        fields = (
            'router',
            'address_family',
            'network',
            'description',
            'comments',
        )


class EIGRPInterfaceForm(NetBoxModelForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    router = DynamicModelChoiceField(
        queryset=EIGRPRouter.objects.all(),
        required=True,
        selector=True,
        label=_('EIGRP Router'),
        query_params={
            'device_id': '$device',
        },
    )
    address_family = DynamicModelChoiceField(
        queryset=EIGRPAddressFamily.objects.all(),
        required=False,
        selector=True,
        label=_('Address Family'),
        query_params={
            'router_id': '$router',
        },
    )
    interface = DynamicModelChoiceField(
        queryset=Interface.objects.all(),
        required=True,
        selector=True,
        label=_('Interface'),
        query_params={
            'device_id': '$device',
        },
    )
    passive = forms.BooleanField(
        required=False,
        label='Passive Interface',
        widget=forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
    )
    comments = CommentField()

    class Meta:
        model = EIGRPInterface
        fields = (
            'device',
            'router',
            'address_family',
            'interface',
            'passive',
            'bfd',
            'authentication',
            'passphrase',
            'description',
            'comments',
        )

        widgets = {
            'bfd': forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance.pk:
            self.initial['device'] = self.instance.interface.device.pk

    def clean(self):
        super().clean()
        if self.cleaned_data.get('router') and self.cleaned_data.get('address_family'):
            if (
                self.cleaned_data.get('router')
                != self.cleaned_data.get('address_family').router
            ):
                raise ValidationError(
                    {
                        'router': _(
                            'EIGRP Router and EIGRP Address Family Router must match'
                        ),
                        'interface': _(
                            'EIGRP Router and EIGRP Address Family Router must match'
                        ),
                    }
                )
        if self.cleaned_data.get('router') and self.cleaned_data.get('interface'):
            if (
                self.cleaned_data.get('router').device
                != self.cleaned_data.get('interface').device
            ):
                raise ValidationError(
                    {
                        'router': _(
                            'EIGRP Router Device and Interface Device must match'
                        ),
                        'interface': _(
                            'EIGRP Interface Device and Interface Device must match'
                        ),
                    }
                )



from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _

from ipam.models import Prefix
from netbox.forms import PrimaryModelForm
from utilities.forms.fields import (
    DynamicModelChoiceField,
    DynamicModelMultipleChoiceField,
)
from utilities.forms.rendering import TabbedGroups, FieldSet

from netbox_routing.models.community import Community, CommunityList
from netbox_routing.models.objects import *

__all__ = (
    'PrefixListForm',
    'PrefixListEntryForm',
    'RouteMapForm',
    'RouteMapEntryForm',
    'ASPathForm',
    'ASPathEntryForm',
    'CustomPrefixForm',
)


class ASPathForm(PrimaryModelForm):

    class Meta:
        model = ASPath
        fields = (
            'name',
            'description',
            'comments',
            'tags',
            'owner',
        )


class ASPathEntryForm(PrimaryModelForm):

    class Meta:
        model = ASPathEntry
        fields = (
            'aspath',
            'sequence',
            'action',
            'pattern',
            'description',
            'comments',
            'tags',
            'owner',
        )


class PrefixListForm(PrimaryModelForm):

    class Meta:
        model = PrefixList
        fields = (
            'name',
            'family',
            'description',
            'comments',
            'tags',
            'owner',
        )

    def clean_family(self):
        family = self.cleaned_data.get('family')
        if not family:
            raise forms.ValidationError(_('Family must be specified'))
        try:
            self.cleaned_data['family'] = int(family)
        except ValueError:
            raise forms.ValidationError(_('Family must be an integer'))
        return self.cleaned_data['family']


class PrefixListEntryForm(PrimaryModelForm):
    ipam_prefix = DynamicModelChoiceField(
        queryset=Prefix.objects.all(),
        required=False,
        selector=True,
        label=_('Prefix'),
    )
    custom_prefix = DynamicModelChoiceField(
        queryset=CustomPrefix.objects.all(),
        required=False,
        selector=True,
        quick_add=True,
        label=_('Custom Prefix'),
    )

    fieldsets = (
        FieldSet(
            'prefix_list',
            'sequence',
            'action',
        ),
        FieldSet(
            TabbedGroups(
                FieldSet('ipam_prefix', name=_('Prefix')),
                FieldSet('custom_prefix', name=_('Custom Prefix')),
            ),
            name=_('Prefix'),
        ),
        FieldSet(
            'le',
            'ge',
            name=_('Filtering'),
        ),
        FieldSet(
            'description',
        ),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = PrefixListEntry
        fields = (
            'prefix_list',
            'sequence',
            'action',
            'ipam_prefix',
            'custom_prefix',
            'le',
            'ge',
            'description',
            'comments',
            'tags',
            'owner',
        )

    def __init__(self, *args, **kwargs):

        # Initialize helper selectors
        instance = kwargs.get('instance')
        initial = kwargs.get('initial', {}).copy()
        if instance:
            if type(instance.assigned_prefix) is Prefix:
                initial['ipam_prefix'] = instance.assigned_prefix
            elif type(instance.assigned_prefix) is CustomPrefix:
                initial['custom_prefix'] = instance.assigned_prefix
        kwargs['initial'] = initial
        super().__init__(*args, **kwargs)

    def clean(self):
        super().clean()
        # Handle object assignment
        selected_objects = [
            field
            for field in ('ipam_prefix', 'custom_prefix')
            if self.cleaned_data[field]
        ]
        if len(selected_objects) > 1:
            raise forms.ValidationError(
                {
                    selected_objects[1]: _(
                        "You can only assign a prefix or a custom prefix."
                    )
                }
            )
        elif selected_objects:
            self.instance.assigned_prefix = self.cleaned_data[selected_objects[0]]
        else:
            raise ValidationError(_('A Prefix or Custom Prefix must be specified'))


class CustomPrefixForm(PrimaryModelForm):

    fieldsets = (
        FieldSet(
            'prefix',
            'description',
        ),
    )

    class Meta:
        model = CustomPrefix
        fields = (
            'prefix',
            'description',
            'comments',
            'tags',
            'owner',
        )


class RouteMapForm(PrimaryModelForm):

    class Meta:
        model = RouteMap
        fields = (
            'name',
            'description',
            'comments',
            'tags',
            'owner',
        )


class RouteMapEntryForm(PrimaryModelForm):
    flow_control = forms.IntegerField(
        required=False,
        label=_("Continue"),
        help_text=_("None = Not enabled; 0 = Next Entry; >0 = Sequence Number"),
    )
    match_community = DynamicModelMultipleChoiceField(
        queryset=Community.objects.all(),
        required=False,
        selector=True,
        label=_('Match Community'),
    )
    match_community_list = DynamicModelMultipleChoiceField(
        queryset=CommunityList.objects.all(),
        required=False,
        selector=True,
        label=_('Match Community List'),
    )
    match_prefix_list = DynamicModelMultipleChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        selector=True,
        label=_('Match Prefix List'),
    )
    match_aspath = DynamicModelMultipleChoiceField(
        queryset=ASPath.objects.all(),
        required=False,
        selector=True,
        label=_('Match AS-Path'),
    )

    fieldsets = (
        FieldSet(
            'route_map',
            'sequence',
            'action',
        ),
        FieldSet(
            TabbedGroups(
                FieldSet('flow_control', name=_('Flow Control')),
                FieldSet(
                    'match_prefix_list',
                    'match_community_list',
                    'match_community',
                    'match_aspath',
                    'match',
                    name=_('Match'),
                ),
                FieldSet('set', name=_('Set')),
            ),
        ),
    )

    class Meta:
        model = RouteMapEntry
        fields = (
            'route_map',
            'sequence',
            'action',
            'flow_control',
            'match_prefix_list',
            'match_community_list',
            'match_community',
            'match_aspath',
            'match',
            'set',
            'description',
            'comments',
            'tags',
            'owner',
        )

    def save(self, commit=True):
        result = super().save(commit)

        self.instance.match_community.set(self.cleaned_data['match_community'])
        self.instance.match_community_list.set(
            self.cleaned_data['match_community_list']
        )
        self.instance.match_prefix_list.set(self.cleaned_data['match_prefix_list'])
        self.instance.match_aspath.set(self.cleaned_data['match_aspath'])

        return result



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable
from netbox_routing.models import EIGRPAddressFamily, EIGRPRouter, EIGRPInterface

__all__ = (
    'EIGRPAddressFamilyTable',
    'EIGRPRouterTable',
    'EIGRPNetworkTable',
    'EIGRPInterfaceTable',
)


class EIGRPRouterTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = EIGRPRouter
        fields = ('pk', 'id', 'name', 'mode', 'pid', 'rid', 'device')
        default_columns = ('pk', 'id', 'name', 'device')


class EIGRPAddressFamilyTable(NetBoxTable):
    router = tables.Column(verbose_name=_('Router'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = EIGRPAddressFamily
        fields = ('pk', 'id', 'family', 'router')
        default_columns = ('pk', 'id', 'family', 'router')


class EIGRPNetworkTable(NetBoxTable):
    router = tables.Column(verbose_name=_('Router'), linkify=True)
    address_family = tables.Column(verbose_name=_('Address Family'), linkify=True)
    network = tables.Column(verbose_name=_('Network'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = EIGRPInterface
        fields = ('pk', 'id', 'router', 'address_family', 'network')
        default_columns = ('pk', 'id', 'router', 'address_family', 'network')


class EIGRPInterfaceTable(NetBoxTable):
    router = tables.Column(verbose_name=_('Router'), linkify=True)
    address_family = tables.Column(verbose_name=_('Address Family'), linkify=True)
    interface = tables.Column(verbose_name=_('Interface'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = EIGRPInterface
        fields = (
            'pk',
            'id',
            'router',
            'address_family',
            'interface',
            'passive',
            'bfd',
            'authentication',
            'passphrase',
        )
        default_columns = ('pk', 'id', 'router', 'address_family', 'interface')



from typing import Annotated, Union

import strawberry
import strawberry_django

from netbox.graphql.types import PrimaryObjectType
from netbox_routing import models
from netbox_routing.graphql.objects.filters import *
from netbox_routing.graphql.community.types import *

__all__ = (
    'ASPathType',
    'ASPathEntryType',
    'PrefixListType',
    'PrefixListEntryType',
    'RouteMapType',
    'RouteMapEntryType',
    'CustomPrefixType',
)


@strawberry_django.type(models.ASPath, fields='__all__', filters=ASPathFilter)
class ASPathType(PrimaryObjectType):

    name: str


@strawberry_django.type(
    models.ASPathEntry,
    fields='__all__',
    filters=ASPathEntryFilter,
    select_related=[
        'aspath',
    ],
)
class ASPathEntryType(PrimaryObjectType):

    aspath: Annotated["ASPathType", strawberry.lazy('netbox_routing.graphql.types')]
    action: str
    sequence: int
    pattern: str | None


@strawberry_django.type(models.PrefixList, fields='__all__', filters=PrefixListFilter)
class PrefixListType(PrimaryObjectType):

    name: str


@strawberry_django.type(
    models.PrefixListEntry,
    fields='__all__',
    exclude=['assigned_prefix_type', 'assigned_prefix_id'],
    filters=PrefixListEntryFilter,
    select_related=[
        'prefix_list',
    ],
)
class PrefixListEntryType(PrimaryObjectType):

    prefix_list: Annotated[
        "PrefixListType", strawberry.lazy('netbox_routing.graphql.types')
    ]
    action: str
    sequence: int
    assigned_prefix_type: (
        Annotated["ContentTypeType", strawberry.lazy('netbox.graphql.types')] | None
    )  # noqa: F821
    assigned_prefix: (
        Union[
            Annotated["PrefixType", strawberry.lazy('ipam.graphql.types')],
            Annotated[
                "CustomPrefixType", strawberry.lazy('netbox_routing.graphql.types')
            ],
        ]
        | None
    )
    le: int | None
    ge: int | None


@strawberry_django.type(models.RouteMap, fields='__all__', filters=RouteMapFilter)
class RouteMapType(PrimaryObjectType):
    name: str


@strawberry_django.type(
    models.RouteMapEntry,
    fields='__all__',
    filters=RouteMapEntryFilter,
    select_related=[
        'route_map',
    ],
)
class RouteMapEntryType(PrimaryObjectType):

    route_map: Annotated[
        "RouteMapType", strawberry.lazy('netbox_routing.graphql.types')
    ]
    action: str
    sequence: int
    match_prefix_list: (
        Annotated["PrefixListType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    match_community_list: (
        Annotated["CommunityListType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    match_community: (
        Annotated["CommunityType", strawberry.lazy('netbox_routing.graphql.types')]
        | None
    )
    match_aspath: (
        Annotated["ASPathType", strawberry.lazy('netbox_routing.graphql.types')] | None
    )


@strawberry_django.type(
    models.CustomPrefix,
    fields='__all__',
    filters=CustomPrefixFilter,
)
class CustomPrefixType(PrimaryObjectType):
    prefix: str



# Generated by Django 5.2.5 on 2025-09-05 23:41

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0015_alter_ospfarea_options_alter_ospfinstance_options'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='ospfinterface',
            name='netbox_routing_ospfinterface_unique_interface',
        ),
        migrations.AlterField(
            model_name='ospfinterface',
            name='interface',
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='ospf_interfaces',
                to='dcim.interface',
            ),
        ),
    ]



from netbox.plugins import PluginMenuItem, PluginMenuButton

__all__ = ('BFD_MENU',)


COL_ADD = 'mdi mdi-plus'
COL_IMPORT = 'mdi mdi-upload'

BFD_MENU = (
    PluginMenuItem(
        link='plugins:netbox_routing:bfdprofile_list',
        link_text='BFD Profile',
        permissions=['netbox_routing.view_bfdprofile'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_routing:bfdprofile_add',
                title='Add',
                icon_class=COL_ADD,
                permissions=['netbox_routing.add_bfdprofile'],
            ),
        ),
    ),
)



from django.utils.translation import gettext as _

from netbox.forms import PrimaryModelForm
from tenancy.forms import TenancyForm
from utilities.forms.rendering import FieldSet

from netbox_routing.models.community import *

__all__ = (
    'CommunityListForm',
    'CommunityListEntryForm',
    'CommunityForm',
)


class CommunityListForm(TenancyForm, PrimaryModelForm):

    fieldsets = (
        FieldSet(
            'name',
        ),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = CommunityList
        fields = [
            'name',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]


class CommunityForm(TenancyForm, PrimaryModelForm):

    fieldsets = (
        FieldSet('community', 'status', 'role'),
        FieldSet('tenant_group', 'tenant', name=_('Tenancy')),
    )

    class Meta:
        model = Community
        fields = [
            'community',
            'status',
            'role',
            'tenant_group',
            'tenant',
            'tags',
            'owner',
        ]


class CommunityListEntryForm(PrimaryModelForm):

    fieldsets = (FieldSet('community_list', 'action', 'community'),)

    class Meta:
        model = CommunityListEntry
        fields = [
            'community_list',
            'action',
            'community',
            'tags',
            'owner',
        ]



from .static import StaticRouteFilterForm
from .bgp import (
    BGPRouterFilterForm,
    BGPScopeFilterForm,
    BGPAddressFamilyFilterForm,
    BGPSettingFilterForm,
    BFDProfileFilterForm,
)
from .ospf import OSPFAreaFilterForm, OSPFInstanceFilterForm, OSPFInterfaceFilterForm
from .objects import (
    PrefixListFilterForm,
    PrefixListEntryFilterForm,
    RouteMapFilterForm,
    RouteMapEntryFilterForm,
)
from .eigrp import *

__all__ = (
    # Static
    'StaticRouteFilterForm',
    # BGP
    'BGPRouterFilterForm',
    'BGPScopeFilterForm',
    'BGPAddressFamilyFilterForm',
    'BGPSettingFilterForm',
    # BFD
    'BFDProfileFilterForm',
    # EIGRP
    'EIGRPRouterFilterForm',
    'EIGRPAddressFamilyFilterForm',
    'EIGRPNetworkFilterForm',
    'EIGRPInterfaceFilterForm',
    # OSPF
    'OSPFAreaFilterForm',
    'OSPFInstanceFilterForm',
    'OSPFInterfaceFilterForm',
    # Routing Objects
    'PrefixListFilterForm',
    'PrefixListEntryFilterForm',
    'RouteMapFilterForm',
    'RouteMapEntryFilterForm',
)



# Generated by Django 5.2.5 on 2026-02-18 19:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0026_denormalized_matches'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bgppeer',
            name='name',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='bgprouter',
            name='name',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]



from django import forms
from django.utils.translation import gettext as _

from ipam.models import Role
from netbox.forms import PrimaryModelBulkEditForm
from netbox_routing.choices import CommunityStatusChoices
from netbox_routing.forms.bulk_edit.base import TenantBulkEditMixin
from utilities.forms import add_blank_choice
from utilities.forms.fields import DynamicModelChoiceField
from utilities.forms.rendering import FieldSet

from netbox_routing.models.community import *


class CommunityBulkEditForm(TenantBulkEditMixin, PrimaryModelBulkEditForm):
    role = DynamicModelChoiceField(
        label=_('Role'),
        queryset=Role.objects.all(),
        required=False,
        selector=True,
    )
    status = forms.ChoiceField(
        label=_('Status'),
        choices=add_blank_choice(CommunityStatusChoices),
        required=False,
    )

    model = Community
    fieldsets = (
        FieldSet(
            'status',
            'role',
            'description',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'role',
        'tenant',
        'description',
    )


class CommunityListBulkEditForm(TenantBulkEditMixin, PrimaryModelBulkEditForm):
    model = CommunityList
    fieldsets = (
        FieldSet(
            'description',
        ),
        FieldSet('tenant', name=_('Tenancy')),
    )
    nullable_fields = (
        'description',
        'tenant',
    )


class CommunityListEntryBulkEditForm(PrimaryModelBulkEditForm):
    community_list = DynamicModelChoiceField(
        label=_('Community List'),
        queryset=CommunityList.objects.all(),
        required=False,
        selector=True,
    )
    community = DynamicModelChoiceField(
        label=_('Community'),
        queryset=Community.objects.all(),
        required=False,
        selector=True,
    )
    model = CommunityListEntry
    fieldsets = (
        FieldSet(
            'community_list',
            'action',
            'community',
            'description',
        ),
    )
    nullable_fields = ('description',)



from django import forms
from django.utils.translation import gettext as _

from dcim.models import Device
from ipam.models import VRF
from netbox.forms import NetBoxModelBulkEditForm
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES
from utilities.forms.fields import (
    DynamicModelChoiceField,
    DynamicModelMultipleChoiceField,
    CommentField,
)
from utilities.forms.rendering import FieldSet

from netbox_routing.models import StaticRoute

__all__ = ('StaticRouteBulkEditForm',)


class StaticRouteBulkEditForm(NetBoxModelBulkEditForm):
    devices = DynamicModelMultipleChoiceField(
        label='Device',
        queryset=Device.objects.all(),
        required=False,
        selector=True,
    )
    vrf = DynamicModelChoiceField(
        label='VRF',
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
    )
    metric = forms.IntegerField(label=_('Metric'), required=False)
    permanent = forms.ChoiceField(
        label=_('Permanent'), choices=BOOLEAN_WITH_BLANK_CHOICES, required=False
    )

    description = forms.CharField(
        label=_('Description'), max_length=200, required=False
    )
    comments = CommentField()

    model = StaticRoute
    fieldsets = (
        FieldSet('devices', 'vrf', 'prefix', 'next_hop', name='Route'),
        FieldSet('metric', 'permanent', name='Attributes'),
        FieldSet(
            'description',
        ),
    )
    nullable_fields = (
        'devices',
        'vrf',
        'metric',
        'permanent',
        'description',
        'comments',
    )



from django import forms
from django.forms import CharField
from django.utils.translation import gettext as _

from dcim.models import Device
from ipam.models import VRF
from netbox.forms import NetBoxModelBulkEditForm
from netbox_routing.models import (
    EIGRPRouter,
    EIGRPAddressFamily,
    EIGRPNetwork,
    EIGRPInterface,
)
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES, add_blank_choice
from utilities.forms.fields import DynamicModelChoiceField, CommentField
from utilities.forms.rendering import FieldSet

from netbox_routing import choices

__all__ = (
    'EIGRPRouterBulkEditForm',
    'EIGRPAddressFamilyBulkEditForm',
    'EIGRPNetworkBulkEditForm',
    'EIGRPInterfaceBulkEditForm',
)


class EIGRPRouterBulkEditForm(NetBoxModelBulkEditForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(), label=_('Device'), required=False, selector=True
    )

    description = forms.CharField(
        label=_('Description'), max_length=200, required=False
    )
    comments = CommentField()

    model = EIGRPRouter
    fieldsets = (
        FieldSet('device', 'mode', name='EIGRP'),
        FieldSet(
            'description',
        ),
    )
    nullable_fields = ()


class EIGRPAddressFamilyBulkEditForm(NetBoxModelBulkEditForm):
    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(), label=_('VRF'), required=False, selector=True
    )
    family = CharField(max_length=4)

    description = forms.CharField(
        label=_('Description'), max_length=200, required=False
    )
    comments = CommentField()

    model = EIGRPAddressFamily
    fieldsets = (FieldSet('description'),)
    nullable_fields = ()


class EIGRPNetworkBulkEditForm(NetBoxModelBulkEditForm):
    router = DynamicModelChoiceField(
        queryset=EIGRPRouter.objects.all(),
        label=_('Router'),
        required=False,
        selector=True,
    )
    address_family = DynamicModelChoiceField(
        queryset=EIGRPAddressFamily.objects.all(),
        label=_('Router'),
        required=False,
        selector=True,
    )

    description = forms.CharField(
        label=_('Description'), max_length=200, required=False
    )
    comments = CommentField()

    model = EIGRPNetwork
    fieldsets = (FieldSet('description'),)
    nullable_fields = ()


class EIGRPInterfaceBulkEditForm(NetBoxModelBulkEditForm):
    router = DynamicModelChoiceField(
        queryset=EIGRPRouter.objects.all(),
        label=_('EIGRP Router'),
        required=False,
        selector=True,
    )
    address_family = DynamicModelChoiceField(
        queryset=EIGRPAddressFamily.objects.all(),
        label=_('EIGRP Address Family'),
        required=False,
        selector=True,
    )
    passive = forms.ChoiceField(
        label=_('Passive'), choices=BOOLEAN_WITH_BLANK_CHOICES, required=False
    )
    bfd = forms.ChoiceField(
        label=_('BFD'), choices=BOOLEAN_WITH_BLANK_CHOICES, required=False
    )
    authentication = forms.ChoiceField(
        label=_('Authentication'),
        choices=add_blank_choice(choices.AuthenticationChoices),
        required=False,
    )
    passphrase = forms.CharField(label=_('Passphrase'), required=False)

    description = forms.CharField(
        label=_('Description'), max_length=200, required=False
    )
    comments = CommentField()

    model = EIGRPInterface
    fieldsets = (
        FieldSet('router', 'address_family', name='EIGRP'),
        FieldSet('priority', 'bfd', 'authentication', 'passphrase', name='Attributes'),
        FieldSet('description'),
    )
    nullable_fields = ()



from django.test import TestCase

# from dcim.models import Device, Interface
# from utilities.testing import create_test_device

# from netbox_routing.forms import *
# from netbox_routing.models import *

__all__ = (
    'EIGRPRouterTestCase',
    'EIGRPAddressFamilyTestCase',
    'EIGRPNetworkTestCase',
    'EIGRPInterfaceTestCase',
)


class EIGRPRouterTestCase(TestCase):
    pass


class EIGRPAddressFamilyTestCase(TestCase):
    pass


class EIGRPNetworkTestCase(TestCase):
    pass


class EIGRPInterfaceTestCase(TestCase):
    pass



# Generated by Django 5.2.5 on 2026-02-17 22:14

from django.db import migrations, models


def get_relations(objects, model):
    m2ms = []
    for pk in objects:
        try:
            object = model.objects.get(pk=pk)
            m2ms.append(object)
        except (model.DoesNotExist, model.MultipleObjectsReturned):
            pass
    return m2ms


def forward_migrate(apps, schema_editor):
    RouteMapEntry = apps.get_model('netbox_routing', 'RouteMapEntry')

    PrefixList = apps.get_model('netbox_routing', 'PrefixList')
    CommunityList = apps.get_model('netbox_routing', 'CommunityList')
    Community = apps.get_model('netbox_routing', 'Community')
    ASPath = apps.get_model('netbox_routing', 'ASPath')

    models = (
        ('ipv4', 'match_prefix_list', PrefixList),
        ('ipv6', 'match_prefix_list', PrefixList),
        ('community_list', 'match_community_list', CommunityList),
        ('community', 'match_community', Community),
        ('aspath', 'match_aspath', ASPath),
    )

    for rme in RouteMapEntry.objects.all():
        match = rme.match
        for key, attr, model in models:
            getattr(rme, attr).set(get_relations(rme.match.get(key), model))
            if key in match.keys():
                del match[key]
        rme.save()


def reverse_migrate(apps, schema_editor):
    RouteMapEntry = apps.get_model('netbox_routing', 'RouteMapEntry')

    for rme in RouteMapEntry.objects.all():
        rme.match['ipv4'] = list(
            rme.match_prefix_list.filter(family=4).values_list('pk', flat=True)
        )
        rme.match['ipv6'] = list(
            rme.match_prefix_list.filter(family=6).values_list('pk', flat=True)
        )
        rme.match['aspath'] = list(rme.match_aspath.values_list('pk', flat=True))
        rme.match['community'] = list(rme.match_community.values_list('pk', flat=True))
        rme.match['community_list'] = list(
            rme.match_community_list.values_list('pk', flat=True)
        )
        rme.save()


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0025_prefixlistentry_prefix_change'),
    ]

    operations = [
        migrations.AddField(
            model_name='routemapentry',
            name='match_aspath',
            field=models.ManyToManyField(
                blank=True, related_name='route_map_entries', to='netbox_routing.aspath'
            ),
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='match_community',
            field=models.ManyToManyField(
                blank=True,
                related_name='route_map_entries',
                to='netbox_routing.community',
            ),
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='match_community_list',
            field=models.ManyToManyField(
                blank=True,
                related_name='route_map_entries',
                to='netbox_routing.communitylist',
            ),
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='match_prefix_list',
            field=models.ManyToManyField(
                blank=True,
                related_name='route_map_entries',
                to='netbox_routing.prefixlist',
            ),
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='flow_control',
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.RunPython(forward_migrate, reverse_code=reverse_migrate),
    ]



from dcim.filtersets import DeviceFilterSet
from dcim.models import Device
from dcim.tables import DeviceTable
from netbox.views.generic import (
    ObjectListView,
    ObjectEditView,
    ObjectView,
    ObjectDeleteView,
    ObjectChildrenView,
    BulkDeleteView,
    BulkEditView,
    BulkImportView,
)
from utilities.views import register_model_view, ViewTab

from netbox_routing.filtersets.static import StaticRouteFilterSet
from netbox_routing.forms import StaticRouteForm
from netbox_routing.forms.bulk_edit import StaticRouteBulkEditForm
from netbox_routing.forms.filtersets.static import StaticRouteFilterForm
from netbox_routing.forms.bulk_import import StaticRouteImportForm
from netbox_routing.models import StaticRoute
from netbox_routing.tables.static import StaticRouteTable

__all__ = (
    'StaticRouteListView',
    'StaticRouteView',
    'StaticRouteDevicesView',
    'StaticRouteEditView',
    'StaticRouteBulkEditView',
    'StaticRouteDeleteView',
    'StaticRouteBulkDeleteView',
    'StaticRouteBulkImportView',
)


@register_model_view(StaticRoute, name='list', path='', detail=False)
class StaticRouteListView(ObjectListView):
    queryset = StaticRoute.objects.all()
    table = StaticRouteTable
    filterset = StaticRouteFilterSet
    filterset_form = StaticRouteFilterForm


@register_model_view(StaticRoute)
class StaticRouteView(ObjectView):
    queryset = StaticRoute.objects.all()
    template_name = 'netbox_routing/staticroute.html'


@register_model_view(StaticRoute, name='devices')
class StaticRouteDevicesView(ObjectChildrenView):
    template_name = 'netbox_routing/staticroute_devices.html'
    queryset = StaticRoute.objects.all()
    child_model = Device
    table = DeviceTable
    filterset = DeviceFilterSet
    tab = ViewTab(
        label='Assigned Devices',
        badge=lambda obj: Device.objects.filter(static_routes=obj).count(),
        permission='dcim.view_device',
        hide_if_empty=True,
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(static_routes=parent)


@register_model_view(StaticRoute, name='add', detail=False)
@register_model_view(StaticRoute, name='edit')
class StaticRouteEditView(ObjectEditView):
    queryset = StaticRoute.objects.all()
    form = StaticRouteForm


@register_model_view(StaticRoute, name='delete')
class StaticRouteDeleteView(ObjectDeleteView):
    queryset = StaticRoute.objects.all()


@register_model_view(StaticRoute, name='bulk_edit', detail=False)
class StaticRouteBulkEditView(BulkEditView):
    queryset = StaticRoute.objects.all()
    filterset = StaticRouteFilterSet
    table = StaticRouteTable
    form = StaticRouteBulkEditForm


@register_model_view(StaticRoute, name='bulk_delete', detail=False)
class StaticRouteBulkDeleteView(BulkDeleteView):
    queryset = StaticRoute.objects.all()
    filterset = StaticRouteFilterSet
    table = StaticRouteTable


@register_model_view(StaticRoute, name='bulk_import', detail=False)
class StaticRouteBulkImportView(BulkImportView):
    queryset = StaticRoute.objects.all()
    model_form = StaticRouteImportForm



# Generated by Django 5.2.4 on 2025-08-02 20:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0014_ospfarea_area_type'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ospfarea',
            options={'ordering': ['area_id']},
        ),
        migrations.AlterModelOptions(
            name='ospfinstance',
            options={'ordering': ['vrf', 'router_id', 'process_id']},
        ),
    ]



from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.test import TestCase

from ipam.models import Role

from netbox_routing.choices import ActionChoices
from netbox_routing.models.community import *

__all__ = (
    'CommunityTestCase',
    'CommunityListTestCase',
    'CommunityListEntryTestCase',
)


class CommunityTestCase(TestCase):
    def setUp(self):
        Role.objects.create(name='Test Role')

    def test_community(self):
        role = Role.objects.get(name='Test Role')
        community_value = '64512'
        community = Community(
            community=community_value,
            status='active',
            role=role,
        )
        community.full_clean()
        community.save()
        self.assertIsInstance(community, Community)
        self.assertEqual(community.__str__(), community_value)

        community_value = '64512:64512'
        community = Community(
            community=community_value,
            status='active',
            role=role,
        )
        community.full_clean()
        community.save()
        self.assertIsInstance(community, Community)
        self.assertEqual(community.__str__(), community_value)

        community_value = '64512:64512:64512'
        community = Community(
            community=community_value,
            status='active',
            role=role,
        )
        community.full_clean()
        community.save()
        self.assertIsInstance(community, Community)
        self.assertEqual(community.__str__(), community_value)

        community_value = '64512:64512:64512:64512'
        community = Community(
            community=community_value,
            status='active',
            role=role,
        )

        with self.assertRaises(ValidationError):
            community.full_clean()
            community.save()

    def test_unique_together(self):

        role = Role.objects.get(name='Test Role')
        community_value = '64512:64512:64512'
        community = Community(
            community=community_value,
            status='active',
            role=role,
        )
        community.full_clean()
        community.save()

        community = Community(
            community=community_value,
            status='active',
            role=role,
        )

        with self.assertRaises(ValidationError):
            community.full_clean()
        with self.assertRaises(IntegrityError):
            community.save()


class CommunityListTestCase(TestCase):
    def setUp(self):
        pass

    def test_community_list(self):
        cl = CommunityList(
            name='Test Community List',
        )
        cl.full_clean()
        cl.save()

    def test_community_list_unique(self):
        cl = CommunityList(
            name='Test Community List',
        )
        cl.full_clean()
        cl.save()

        cl = CommunityList(
            name='Test Community List',
        )
        with self.assertRaises(ValidationError):
            cl.full_clean()
        with self.assertRaises(IntegrityError):
            cl.save()


class CommunityListEntryTestCase(TestCase):
    def setUp(self):
        role = Role.objects.create(name='Test Role')
        self.communities = (
            Community(
                community='64512',
                status='active',
                role=role,
            ),
            Community(
                community='64513',
                status='active',
                role=role,
            ),
            Community(
                community='64514',
                status='active',
                role=role,
            ),
            Community(
                community='64515',
                status='active',
                role=role,
            ),
            Community(
                community='64516',
                status='active',
                role=role,
            ),
        )
        Community.objects.bulk_create(self.communities)

        self.cl = CommunityList(
            name='Test Community List',
        )
        self.cl.full_clean()
        self.cl.save()

    def test_community_list_entry(self):
        for community in self.communities:
            cle = CommunityListEntry(
                community_list=self.cl, action=ActionChoices.PERMIT, community=community
            )
            cle.full_clean()
            cle.save()



# Generated by Django 5.2.5 on 2026-02-13 21:04

import django.core.validators
import django.db.models.deletion
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


def forward_migrate(apps, schema_editor):
    BGPPeer = apps.get_model('netbox_routing', 'BGPPeer')
    BGPSessionTemplate = apps.get_model('netbox_routing', 'BGPSessionTemplate')
    BFDProfile = apps.get_model('netbox_routing', 'BFDPRofile')

    for peer in BGPPeer.objects.all():
        if peer.bfd_enabled:
            try:
                bfd = BFDProfile.objects.get(name='BFD Enabled (default profile)')
            except BFDProfile.DoesNotExist:
                bfd = BFDProfile.objects.create(name='BFD Enabled (default profile)')
            peer.bfd = bfd
            peer.save()

    for peer in BGPSessionTemplate.objects.all():
        if peer.bfd_enabled:
            try:
                bfd = BFDProfile.objects.get(name='BFD Enabled (default profile)')
            except BFDProfile.DoesNotExist:
                bfd = BFDProfile.objects.create(name='BFD Enabled (default profile)')
            peer.bfd = bfd
            peer.save()


def reverse_migrate(apps, schema_editor):
    BFDProfile = apps.get_model('netbox_routing', 'BFDPRofile')
    for profile in BFDProfile.objects.all():
        for peer in profile.bgp_peers.all():
            peer.bfd_enabled = True
            peer.save()
        for peer in profile.bgp_session_templates.all():
            peer.bfd_enabled = True
            peer.save()


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0134_owner'),
        ('netbox_routing', '0023_peer_and_session_add_ttl'),
        ('tenancy', '0022_add_comments_to_organizationalmodel'),
        ('users', '0015_owner'),
    ]

    operations = [
        migrations.CreateModel(
            name='BFDProfile',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100)),
                (
                    'min_tx_int',
                    models.PositiveIntegerField(
                        blank=True,
                        null=True,
                    ),
                ),
                (
                    'min_rx_int',
                    models.PositiveIntegerField(
                        blank=True,
                        null=True,
                    ),
                ),
                (
                    'multiplier',
                    models.PositiveSmallIntegerField(
                        blank=True,
                        null=True,
                    ),
                ),
                (
                    'hold',
                    models.PositiveIntegerField(
                        blank=True,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(60),
                            django.core.validators.MaxValueValidator(60000),
                        ],
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bfd_profiles',
                        to='tenancy.tenant',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BFD Profile',
                'verbose_name_plural': 'BFD Profiles',
                'ordering': ('name',),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.RenameField(
            model_name='bgppeer',
            old_name='bfd',
            new_name='bfd_enabled',
        ),
        migrations.RenameField(
            model_name='bgpsessiontemplate',
            old_name='bfd',
            new_name='bfd_enabled',
        ),
        migrations.AddField(
            model_name='bgppeer',
            name='bfd',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name='bgp_peers',
                to='netbox_routing.bfdprofile',
            ),
        ),
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='bfd',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name='bgp_session_templates',
                to='netbox_routing.bfdprofile',
            ),
        ),
        migrations.AddConstraint(
            model_name='bfdprofile',
            constraint=models.UniqueConstraint(
                fields=('name',), name='netbox_routing_bfdprofile_name'
            ),
        ),
        migrations.RunPython(code=forward_migrate, reverse_code=reverse_migrate),
        migrations.AlterField(
            model_name='bfdprofile',
            name='min_rx_int',
            field=models.PositiveIntegerField(
                default=0,
                validators=[
                    django.core.validators.MinValueValidator(60),
                    django.core.validators.MaxValueValidator(60000),
                ],
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='bfdprofile',
            name='min_tx_int',
            field=models.PositiveIntegerField(
                default=0,
                validators=[
                    django.core.validators.MinValueValidator(60),
                    django.core.validators.MaxValueValidator(60000),
                ],
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='bfdprofile',
            name='multiplier',
            field=models.PositiveSmallIntegerField(
                default=0,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(255),
                ],
            ),
            preserve_default=False,
        ),
        migrations.RemoveField(
            model_name='bgppeer',
            name='bfd_enabled',
        ),
        migrations.RemoveField(
            model_name='bgpsessiontemplate',
            name='bfd_enabled',
        ),
    ]



from utilities.choices import ChoiceSet

__all__ = ('AuthenticationChoices',)


class AuthenticationChoices(ChoiceSet):
    KEYCHAIN = 'key-chain'
    MESSAGE_DIGEST = 'message-digest'
    NULL = 'null'

    CHOICES = [
        (KEYCHAIN, 'Key Chain'),
        (MESSAGE_DIGEST, 'Message Digest'),
        (NULL, 'Null Authentication'),
    ]



from netbox.api.routers import NetBoxRouter
from .views import *
from .views.objects import CustomPrefixViewSet

router = NetBoxRouter()
router.register('routes/static', StaticRouteViewSet)
router.register('bgp/setting', BGPSettingViewSet)
router.register('bgp/peer-template', BGPPeerTemplateViewSet)
router.register('bgp/policy-template', BGPPolicyTemplateViewSet)
router.register('bgp/session-template', BGPSessionTemplateViewSet)
router.register('bgp/router', BGPRouterViewSet)
router.register('bgp/scope', BGPScopeViewSet)
router.register('bgp/address-family', BGPAddressFamilyViewSet)
router.register('bgp/peer', BGPPeerViewSet)
router.register('bgp/peer-address-family', BGPPeerAddressFamilyViewSet)
router.register('bfd/profile', BFDProfileViewSet)
router.register('ospf/instance', OSPFInstanceViewSet)
router.register('ospf/area', OSPFAreaViewSet)
router.register('ospf/interface', OSPFInterfaceViewSet)
router.register('eigrp/router', EIGRPRouterViewSet)
router.register('eigrp/address-family', EIGRPAddressFamilyViewSet)
router.register('eigrp/network', EIGRPNetworkViewSet)
router.register('eigrp/interface', EIGRPInterfaceViewSet)
router.register('objects/custom-prefix', CustomPrefixViewSet)
router.register('objects/prefix-list', PrefixListViewSet)
router.register('objects/prefix-list-entry', PrefixListEntryViewSet)
router.register('objects/route-map', RouteMapViewSet)
router.register('objects/route-map-entry', RouteMapEntryViewSet)
router.register('objects/as-path', ASPathViewSet)
router.register('objects/as-path-entry', ASPathEntryViewSet)
router.register('objects/community-list', CommunityListViewSet)
router.register('objects/community-list-entry', CommunityListEntryViewSet)
router.register('objects/community', CommunityViewSet)
urlpatterns = router.urls



# Generated by Django 5.2.5 on 2026-02-13 19:44

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0022_peer_add_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='bgppeer',
            name='ttl',
            field=models.PositiveSmallIntegerField(
                blank=True,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(255),
                ],
            ),
        ),
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='ttl',
            field=models.PositiveSmallIntegerField(
                blank=True,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(255),
                ],
            ),
        ),
    ]



from django.contrib.contenttypes.models import ContentType
from drf_spectacular.utils import extend_schema_field
from rest_framework import serializers

from ipam.api.serializers_.asns import ASNSerializer
from ipam.api.serializers_.ip import IPAddressSerializer
from ipam.api.serializers_.vrfs import VRFSerializer
from netbox.api.fields import ContentTypeField
from netbox.api.gfk_fields import GFKSerializerField
from netbox.api.serializers import NetBoxModelSerializer
from tenancy.api.serializers_.tenants import TenantSerializer
from utilities.api import get_serializer_for_model

from netbox_routing.constants.bgp import *
from netbox_routing.models.bgp import *

__all__ = (
    'BGPSettingSerializer',
    'BGPRouterSerializer',
    'BGPScopeSerializer',
    'BGPAddressFamilySerializer',
    'BGPPeerSerializer',
    'BGPPeerTemplateSerializer',
    'BGPPolicyTemplateSerializer',
    'BGPSessionTemplateSerializer',
    'BGPPeerAddressFamilySerializer',
    'BFDProfileSerializer',
)


class BGPSettingSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgpsetting-detail'
    )

    assigned_object = GFKSerializerField(read_only=True)

    class Meta:
        model = BGPSetting
        fields = (
            'url',
            'id',
            'display',
            'assigned_object_type',
            'assigned_object_id',
            'assigned_object',
            'key',
            'value',
            'description',
            'comments',
            'custom_fields',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'assigned_object',
            'key',
        )

    @extend_schema_field(serializers.JSONField(allow_null=True))
    def get_assigned_object(self, obj):
        if obj.assigned_object is None:
            return None
        serializer = get_serializer_for_model(obj.assigned_object)
        context = {'request': self.context['request']}
        return serializer(obj.assigned_object, context=context, nested=True).data


class BGPSessionTemplateSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgpscope-detail'
    )

    remote_as = ASNSerializer(nested=True, required=False)
    local_as = ASNSerializer(nested=True, required=False)
    tenant = TenantSerializer(nested=True, required=False)

    class Meta:
        model = BGPSessionTemplate
        fields = (
            'url',
            'id',
            'display',
            'name',
            'parent',
            'enabled',
            'remote_as',
            'local_as',
            'bfd',
            'ttl',
            'password',
            'tenant',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
            'parent',
        )


class BGPPolicyTemplateSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgpscope-detail'
    )

    tenant = TenantSerializer(nested=True, required=False)

    class Meta:
        model = BGPPolicyTemplate
        fields = (
            'url',
            'id',
            'display',
            'name',
            'enabled',
            'prefixlist_in',
            'prefixlist_out',
            'routemap_in',
            'routemap_out',
            'tenant',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
        )


class BGPPeerTemplateSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgpaddressfamily-detail'
    )

    remote_as = ASNSerializer(nested=True, required=False)
    tenant = TenantSerializer(nested=True, required=False)

    class Meta:
        model = BGPPeerTemplate
        fields = (
            'url',
            'id',
            'display',
            'name',
            'remote_as',
            'enabled',
            'tenant',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
            'remote_as',
        )


class BGPRouterSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgprouter-detail'
    )

    assigned_object = GFKSerializerField(read_only=True)
    asn = ASNSerializer(nested=True)
    settings = BGPSettingSerializer(many=True, required=False)
    tenant = TenantSerializer(nested=True, required=False)
    peer_templates = BGPSessionTemplateSerializer(
        many=True, nested=True, required=False
    )
    policy_templates = BGPSessionTemplateSerializer(
        many=True, nested=True, required=False
    )
    session_templates = BGPSessionTemplateSerializer(
        many=True, nested=True, required=False
    )

    class Meta:
        model = BGPRouter
        fields = (
            'url',
            'id',
            'display',
            'name',
            'asn',
            'assigned_object',
            'assigned_object_type',
            'assigned_object_id',
            'session_templates',
            'policy_templates',
            'peer_templates',
            'settings',
            'tenant',
            'description',
            'comments',
        )
        brief_fields = ('url', 'id', 'display', 'asn', 'assigned_object', 'name')

    @extend_schema_field(serializers.JSONField(allow_null=True))
    def get_assigned_object(self, obj):
        if obj.assigned_object is None:
            return None
        serializer = get_serializer_for_model(obj.assigned_object)
        context = {'request': self.context['request']}
        return serializer(obj.assigned_object, context=context, nested=True).data


class BGPScopeSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgpscope-detail'
    )

    router = BGPRouterSerializer(nested=True)
    vrf = VRFSerializer(nested=True)
    settings = BGPSettingSerializer(many=True, required=False)
    tenant = TenantSerializer(nested=True, required=False)

    class Meta:
        model = BGPScope
        fields = (
            'url',
            'id',
            'display',
            'router',
            'vrf',
            'settings',
            'tenant',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'router',
            'vrf',
        )


class BGPAddressFamilySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgpaddressfamily-detail'
    )

    scope = BGPScopeSerializer(nested=True)
    settings = BGPSettingSerializer(many=True, required=False)

    class Meta:
        model = BGPAddressFamily
        fields = (
            'url',
            'id',
            'display',
            'scope',
            'address_family',
            'settings',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'scope',
            'address_family',
        )


class BGPPeerSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgppeer-detail'
    )

    scope = BGPScopeSerializer(nested=True)
    peer = IPAddressSerializer(nested=True)
    source = IPAddressSerializer(nested=True, required=False)
    remote_as = ASNSerializer(nested=True, required=False)
    local_as = ASNSerializer(nested=True, required=False)
    tenant = TenantSerializer(nested=True, required=False)

    class Meta:
        model = BGPPeer
        fields = (
            'url',
            'id',
            'display',
            'scope',
            'peer',
            'source',
            'name',
            'remote_as',
            'local_as',
            'status',
            'enabled',
            'bfd',
            'ttl',
            'password',
            'tenant',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
            'scope',
            'peer',
            'remote_as',
            'enabled',
            'status',
        )


class BGPPeerAddressFamilySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bgppeeraddressfamily-detail'
    )
    assigned_object_type = ContentTypeField(
        queryset=ContentType.objects.filter(BGPPEERAF_ASSIGNMENT_MODELS),
        required=False,
        allow_null=True,
    )
    assigned_object = GFKSerializerField(read_only=True)
    # tenant = TenantSerializer(nested=True)

    class Meta:
        model = BGPPeerAddressFamily
        fields = (
            'url',
            'id',
            'display',
            'assigned_object_type',
            'assigned_object_id',
            'assigned_object',
            'address_family',
            'enabled',
            'peer_policy',
            'prefixlist_in',
            'prefixlist_out',
            'routemap_in',
            'routemap_out',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'assigned_object',
            'address_family',
            'enabled',
        )

    @extend_schema_field(serializers.JSONField(allow_null=True))
    def get_assigned_object(self, obj):
        if obj.assigned_object is None:
            return None
        serializer = get_serializer_for_model(obj.assigned_object)
        context = {'request': self.context['request']}
        return serializer(obj.assigned_object, context=context, nested=True).data


class BFDProfileSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:bfdprofile-detail'
    )
    tenant = TenantSerializer(nested=True, required=False)

    class Meta:
        model = BFDProfile
        fields = (
            'url',
            'id',
            'display',
            'description',
            'comments',
            'name',
            'min_rx_int',
            'min_tx_int',
            'multiplier',
            'hold',
            'tenant',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
        )



# Generated by Django 5.0.8 on 2024-09-28 03:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0010_eigrp'),
    ]

    operations = [
        migrations.AddField(
            model_name='ospfinterface',
            name='passive',
            field=models.BooleanField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.0.3 on 2022-04-01 18:02

import django.core.serializers.json
from django.db import migrations, models
import django.db.models.deletion
import ipam.fields
import netbox_routing.fields.ip
import taggit.managers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('ipam', '0057_created_datetimefield'),
        ('extras', '0072_created_datetimefield'),
        ('dcim', '0153_created_datetimefield'),
    ]

    operations = [
        migrations.CreateModel(
            name='PrefixList',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RouteMap',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='StaticRoute',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                    ),
                ),
                ('prefix', ipam.fields.IPNetworkField()),
                ('next_hop', netbox_routing.fields.ip.IPAddressField()),
                ('name', models.CharField(blank=True, max_length=50, null=True)),
                ('metric', models.PositiveSmallIntegerField()),
                ('permanent', models.BooleanField()),
                (
                    'devices',
                    models.ManyToManyField(
                        related_name='static_routes', to='dcim.device'
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'vrf',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='staticroutes',
                        to='ipam.vrf',
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='RouteMapEntry',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                    ),
                ),
                ('type', models.CharField(max_length=6)),
                ('sequence', models.PositiveSmallIntegerField()),
                (
                    'route_map',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='entries',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PrefixListEntry',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                    ),
                ),
                ('sequence', models.PositiveSmallIntegerField()),
                ('type', models.CharField(max_length=6)),
                ('prefix', ipam.fields.IPNetworkField()),
                ('ge', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('le', models.PositiveSmallIntegerField(blank=True, null=True)),
                (
                    'prefix_list',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='entries',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddConstraint(
            model_name='staticroute',
            constraint=models.CheckConstraint(
                check=models.Q(models.Q(('metric__lte', 255), ('metric__gte', 0))),
                name='metric_gte_lte',
            ),
        ),
    ]



from django.contrib.contenttypes.fields import GenericRelation, GenericRel


class NetBoxGenericRel(GenericRel):
    def value_from_object(self, obj):
        """Return the value of this field in the given model instance."""
        if value := getattr(obj, self.related_name).first():
            return value.pk
        return None


class NetBoxGenericRelation(GenericRelation):
    rel_class = NetBoxGenericRel



from netbox.plugins import PluginMenuButton, PluginMenuItem

__all__ = ('BGP_MENU',)


COL_ADD = 'mdi mdi-plus'
COL_IMPORT = 'mdi mdi-upload'


def build_menus():
    menus = []
    menu_items = (
        ('bgpsetting', 'Settings'),
        ('bgppeertemplate', 'Peer Templates'),
        ('bgppolicytemplate', 'Policy Templates'),
        ('bgpsessiontemplate', 'Session Templates'),
        ('bgprouter', 'Routers'),
        ('bgpscope', 'Scopes'),
        ('bgpaddressfamily', 'Address Families'),
        ('bgppeer', 'Peers'),
        ('bgppeeraddressfamily', 'Peer Address Families'),
    )
    for model, name in menu_items:
        menu = PluginMenuItem(
            link=f'plugins:netbox_routing:{model}_list',
            link_text=f'{name}',
            permissions=[f'netbox_routing.view_{model}'],
            buttons=(
                PluginMenuButton(
                    link=f'plugins:netbox_routing:{model}_add',
                    title='Add',
                    icon_class=COL_ADD,
                    permissions=[f'netbox_routing.add_{model}'],
                ),
                # PluginMenuButton('plugins:netbox_routing:bgprouter_bulk_import', 'Import', COL_IMPORT),
            ),
        )
        menus.append(menu)

    return tuple(menus)


BGP_MENU = build_menus()



import django_filters
from django.db.models import Q
from django.utils.translation import gettext as _

from netbox.filtersets import NetBoxModelFilterSet
from netbox_routing.models import RouteMapEntry
from tenancy.filtersets import TenancyFilterSet
from ipam.models import Role

from utilities.filtersets import register_filterset

from netbox_routing.models.community import *

__all__ = (
    'CommunityFilterSet',
    'CommunityListFilterSet',
    'CommunityListEntryFilterSet',
)


@register_filterset
class CommunityFilterSet(TenancyFilterSet, NetBoxModelFilterSet):
    role_id = django_filters.ModelMultipleChoiceFilter(
        field_name='role',
        queryset=Role.objects.all(),
        label=_('Role (ID)'),
    )
    role = django_filters.ModelMultipleChoiceFilter(
        field_name='role__name',
        queryset=Role.objects.all(),
        to_field_name='name',
        label=_('Role (Name)'),
    )
    route_map_entry_id = django_filters.ModelMultipleChoiceFilter(
        field_name='route_map_entries',
        queryset=RouteMapEntry.objects.all(),
        label=_('Route Map Entry (ID)'),
    )

    class Meta:
        model = Community
        fields = (
            'community',
            'status',
            'role_id',
            'role',
            'tenant_id',
            'tenant',
            'tenant_group_id',
            'tenant_group',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(community__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class CommunityListFilterSet(TenancyFilterSet, NetBoxModelFilterSet):
    route_map_entry_id = django_filters.ModelMultipleChoiceFilter(
        field_name='route_map_entries',
        queryset=RouteMapEntry.objects.all(),
        label=_('Route Map Entry (ID)'),
    )

    class Meta:
        model = CommunityList
        fields = ('name', 'tenant_id', 'tenant', 'tenant_group_id', 'tenant_group')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)
        return queryset.filter(qs_filter).distinct()


@register_filterset
class CommunityListEntryFilterSet(NetBoxModelFilterSet):
    community_list_id = django_filters.ModelMultipleChoiceFilter(
        field_name='community_list',
        queryset=CommunityList.objects.all(),
        label=_('Community List (ID)'),
    )
    community_list = django_filters.ModelMultipleChoiceFilter(
        field_name='community_list__name',
        queryset=CommunityList.objects.all(),
        to_field_name='name',
        label=_('Community List (Name)'),
    )
    community_id = django_filters.ModelMultipleChoiceFilter(
        field_name='community',
        queryset=Community.objects.all(),
        label=_('Community (ID)'),
    )
    community = django_filters.ModelMultipleChoiceFilter(
        field_name='community__community',
        queryset=Community.objects.all(),
        to_field_name='community',
        label=_('Community (Community)'),
    )

    class Meta:
        model = CommunityListEntry
        fields = (
            'community_list_id',
            'community_list',
            'community_id',
            'community',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(community_list__name__icontains=value)
        qs_filter |= Q(community__community__icontains=value)
        return queryset.filter(qs_filter).distinct()



from typing import Annotated

import strawberry
import strawberry_django
from strawberry import ID
from strawberry_django import FilterLookup

from netbox.graphql.filters import PrimaryModelFilter
from netbox_routing import models
from netbox_routing.graphql.filter_mixins import VRFMixin, DeviceMixin, InterfaceMixin

__all__ = (
    'OSPFAreaFilter',
    'OSPFInstanceFilter',
    'OSPFInterfaceFilter',
)


@strawberry_django.filter(models.OSPFInstance, lookups=True)
class OSPFInstanceFilter(VRFMixin, DeviceMixin, PrimaryModelFilter):
    router_id: FilterLookup[str] | None = strawberry_django.filter_field()


@strawberry_django.filter(models.OSPFArea, lookups=True)
class OSPFAreaFilter(PrimaryModelFilter):
    area_id: FilterLookup[str] | None = strawberry_django.filter_field()


@strawberry_django.filter(models.OSPFInterface, lookups=True)
class OSPFInterfaceFilter(InterfaceMixin, VRFMixin, PrimaryModelFilter):
    instance: (
        Annotated[
            'OSPFInstanceFilter', strawberry.lazy('netbox_routing.graphql.filters')
        ]
        | None
    ) = strawberry_django.filter_field()
    instance_id: ID | None = strawberry_django.filter_field()
    area: (
        Annotated['OSPFAreaFilter', strawberry.lazy('netbox_routing.graphql.filters')]
        | None
    ) = strawberry_django.filter_field()
    area_id: ID | None = strawberry_django.filter_field()
